# $Id$
# experimental element colorizer.

try:
    import cElementTree as ET
except ImportError:
    import elementtree.ElementTree as ET

import colorizer
import cStringIO

##
# A colorizer that converts a code snippet to a colorized XHTML element
# tree.

class ElementColorizer(colorizer.Colorizer):

    def __init__(self, text):
        self.readline = cStringIO.StringIO(text).readline
        self.elem_root = ET.Element("pre")
        self.elem_style = None

    def colorize(self):
        colorizer.Colorizer.colorize(self)
        return self.elem_root

    def make_span(self, class_):
        e = ET.SubElement(self.elem_root, "span")
        e.set("class", class_)
        return e

    def lineno(self, lineno):
        e = self.make_span("lineno")
        e = ET.SubElement(e, "a")
        e.attrib["name"] = str(lineno)
        e.text = "%05d" % lineno
        e.tail = " "

    def write(self, text):
        if self.elem_style is not None:
            elem = self.elem_style
        elif len(self.elem_root):
            elem = self.elem_root[-1]
            elem.tail = (elem.tail or "") + text
            return
        else:
            elem = self.elem_root
        elem.text = (elem.text or "") + text

    def style(self, style, token):
        if self.elem_style is not None:
            current = self.elem_style.get("class")
        else:
            current = None
        if style != current:
            if style:
                self.elem_style = self.make_span(style)
            else:
                self.elem_style = None
        return current

if __name__ == "__main__":
    import sys
    PYTHONWORKS_STYLE = """
    .class {color:firebrick;font-weight:bold;}
    .comment {color:darkolivegreen;font-style:italic;}
    .decorator {color:firebrick;}
    .function {color:firebrick;font-weight:bold;}
    .keyword {color:firebrick;}
    .lineno {color:gray;border-right:1px solid gray;padding-right:5px;}
    .string {color:navy;}
    """
    try:
        file = sys.argv[1]
    except IndexError:
        file = "pythondoc.py"
    c = ElementColorizer(open(file).read())
    import time
    t0 = time.clock()
    tree = c.colorize()
    t0 = time.clock() - t0
    print "<style>", PYTHONWORKS_STYLE, "</style>"
    ET.dump(tree)
    print >>sys.stderr, t0




import turbogears
from turbogears.i18n import sogettext
from turbogears.i18n.tg_gettext import tg_gettext
from turbogears.i18n.tests import setup_module as basic_setup_module

from turbogears.i18n.tests.test_tg_gettext import test_gettext, test_ngettext
from turbogears.i18n.tests.test_tg_gettext import test_invalid_domain
from turbogears.i18n.tests.test_kidutils import test_match_template, test_i18n_filter

def setup_module(): 
    turbogears.config.update({"i18n.gettext":sogettext.so_gettext})
    basic_setup_module()

def teardown_module(): 
    turbogears.config.update({"i18n.gettext":tg_gettext})

def test_so_gettext():
    test_gettext()
    test_ngettext()
    test_match_template()
    test_i18n_filter()
    test_invalid_domain()




# test-file: test_command_i18n.py

"""Command-line user interface for i18n administration."""

import re
import glob
import os
import os.path
import atexit
import optparse
import tempfile

from elementtree.ElementTree import ElementTree

import formencode
import turbogears
import turbogears.i18n
from turbogears.toolbox.admi18n import pygettext, msgfmt, catalog
from turbogears.toolbox.admi18n.catalog import quote, normalize
from turbogears.command.base import silent_os_remove
from turbogears.util import get_model, load_project_config, get_package_name
from pkg_resources import resource_filename

class ProgramError(StandardError):
    """Signals about a general application error."""

def copy_file(src, dest):
    if os.path.exists(dest):
        os.remove(dest)
    data = open(src, 'rb').read()
    open(dest, 'wb').write(data)

class InternationalizationTool:
    "Manages i18n data via command-line interface."

    desc = "Manage i18n data"
    need_project = True
    config = False

    name = None
    package = None
    __version__ = "0.2"
    __author__ = "Max Ischenko, U{http://maxischenko.in.ua}"
    __email__ = "ischenko@gmail.com"
    __copyright__ = "Copyright 2005-2006 Max Ischenko"
    __license__ = "MIT"

    def __init__(self, version):
        parser = optparse.OptionParser(usage="""
%prog [options] <command>

Available commands:
  add <locale>  Creates a message catalog for specified locale
  collect       Scan source files to gather translatable strings in a .pot file
  merge         Sync message catalog in different languages with .pot file
  compile       Compile message catalog (.po -> .mo)
  clean         Delete backups and compiled files
""", version="%prog " + self.__version__)
        parser.add_option("-f", "--force", default=False,
                action="store_true", dest="force_ops",
                help="Force potentially damaging actions")
        parser.add_option("-a", "--ascii", default=False,
                action="store_true", dest="ascii_output",
                help="Escape non-ascii characters")
        parser.add_option("-K", "--no-kid-support", default=True,
                action="store_false", dest="kid_support",
                help="Do not extract messages from Kid templates")
        parser.add_option("", "--loose-kid-support",
                action="store_true", dest="loose_kid_support",
                help="Extract ALL messages from Kid templates" \
                     " (this is default)")
        parser.add_option("", "--strict-kid-support",
                action="store_false", dest="loose_kid_support",
                help="Extract only messages marked with lang attribute " \
                     "from Kid templates")
        parser.add_option("", "--src-dir", default=None,
                action="store", dest="source_dir",
                help="Directory that contains source files")
        parser.set_defaults(loose_kid_support=True)
        self.parser = parser

    def load_project_config(self):
        """Chooses the config file, trying to guess whether this is a
        development or installed project."""

        # defaults
        self.locale_dir = 'locales'
        self.domain = 'messages'

        # check whether user specified custom settings
        if self.config:
            load_project_config()
 
        if turbogears.config.get("i18n.locale_dir"):
            self.locale_dir = turbogears.config.get("i18n.locale_dir")
            print 'Use %s as a locale directory' % self.locale_dir
        if turbogears.config.get('i18n.domain'):
            self.domain = turbogears.config.get("i18n.domain")
            print 'Use %s as a message domain' % self.domain

        if os.path.exists(self.locale_dir) and \
                not os.path.isdir(self.locale_dir):
                    raise ProgramError, \
                            ('%s is not a directory' % self.locale_dir)

        if not os.path.exists(self.locale_dir):
            os.makedirs(self.locale_dir)

    def parse_args(self):
        return self.parser.parse_args()

    def run(self):
        self.load_project_config()
        (options, args) = self.parse_args()
        if not args:
            self.parser.error("No command specified")
        self.options = options
        command, args = args[0], args[1:]
        if 'collect' == command:
            self.scan_source_files()
        elif 'add' == command:
            self.add_languages(args)
        elif 'compile' == command:
            self.compile_message_catalogs()
        elif 'merge' == command:
            self.merge_message_catalogs()
        elif 'clean' == command:
            self.clean_generated_files()
        else:
            self.parser.error("Command not recognized")
    def clean_generated_files(self):
        potfile = self.get_potfile_path()
        silent_os_remove(potfile.replace('.pot', '.bak'))
        for fname in self.list_message_catalogs():
            silent_os_remove(fname.replace('.po', '.mo'))
            silent_os_remove(fname.replace('.po', '.back'))
    def merge_message_catalogs(self):
        potfile = self.get_potfile_path()
        catalogs = self.list_message_catalogs()
        catalog.merge(potfile, catalogs)
    def compile_message_catalogs(self):
        for fname in self.list_message_catalogs():
            dest = fname.replace('.po','.mo')
            msgfmt.make(fname, dest)
            if os.path.exists(dest):
                print 'Compiled %s OK' % fname
            else:
                print 'Compilation of %s failed!' % fname

    def _copy_file_withcheck(self, sourcefile, targetfile):
        if not (os.path.exists(targetfile) and not self.options.force_ops):
            copy_file(sourcefile, targetfile)
            print 'Copy', sourcefile, 'to', targetfile
        else:
            print "File %s exists, use --force to override" % targetfile
            

    def _copy_moduletranslation(self, sourcefile, targetdir, language):
        modulefilename = os.path.basename(sourcefile)
        if os.path.exists(sourcefile):
            targetfile = os.path.join(targetdir, modulefilename)
            self._copy_file_withcheck(sourcefile, targetfile)
        else:
            print "%s translation for language '%s' does not exist (file searched '%s').\nPlease consider to contribute a translation." % (modulefilename, language, sourcefile)


    def add_languages(self, codes):
        potfile = self.get_potfile_path()
        if not os.path.isfile(potfile):
            print "Run 'collect' first to create", potfile
            return
        for code in codes:
            catalog_file = self.get_locale_catalog(code)
            langdir = os.path.dirname(catalog_file)
            if not os.path.exists(langdir):
                os.makedirs(langdir)

            sourcefile_fe = os.path.join(formencode.api.get_localedir(), code, \
                "LC_MESSAGES","FormEncode.mo")
            self._copy_moduletranslation(sourcefile_fe, langdir, code)

            basedir_i18n_tg = resource_filename("turbogears.i18n", "data")
            sourcefile_tg  = os.path.join(basedir_i18n_tg, code, \
                                          "LC_MESSAGES", "TurboGears.mo")
            self._copy_moduletranslation(sourcefile_tg, langdir, code)

            self._copy_file_withcheck(potfile, catalog_file)
                

        
        
    def scan_source_files(self):
        source_files = []
        kid_files = []
        srcdir = self.options.source_dir or get_package_name()
        print 'Scanning source directory', srcdir
        for root, dirs, files in os.walk(srcdir):
            if os.path.basename(root).lower() in ('CVS', '.svn'):
                continue
            for fname in files:
                name,ext = os.path.splitext(fname)
                srcfile = os.path.join(root, fname)
                if ext == '.py':
                    source_files.append(srcfile)
                elif ext == '.kid':
                    kid_files.append(srcfile)
                    #print 'add', srcfile
                else:
                    pass # do nothing
        (tmp_handle, tmp_potfile) = tempfile.mkstemp('.pot', 'tmp', self.locale_dir)
        os.close(tmp_handle)
        potbasename = os.path.basename(tmp_potfile)[:-4]
        pygettext_options = ['-v', '-d', potbasename, \
                '-p', os.path.dirname(tmp_potfile)]
        if self.options.ascii_output:
            pygettext_options.insert(0, '-E')
        pygettext.sys.argv = [''] + pygettext_options + source_files
        pygettext.main()
        if not os.path.exists(tmp_potfile):
            raise ProgramError, 'pygettext failed'
        atexit.register(silent_os_remove, tmp_potfile)
        if kid_files and self.options.kid_support:
            self.scan_kid_files(tmp_potfile, kid_files)
        potfile = self.get_potfile_path()
        if os.path.isfile(potfile):
            bakfile = potfile.replace('.pot', '.bak')
            silent_os_remove(bakfile)
            os.rename(potfile, bakfile)
            print 'Backup existing file to', bakfile
        os.rename(tmp_potfile, potfile)
        print 'Message templates written to', potfile
    def scan_kid_files(self, potfile, files):
        messages = []
        tags_to_ignore = ['script', 'style']
        keys = []
        kid_expr_re = re.compile(r"_\(('(?P<texta>[^']*)'|\"(?P<textb>[^\"]*)\")\)")
        for fname in files:
            print 'working on', fname
            tree = None
            try:
                tree = ElementTree(file=fname).getroot()
            except Exception, e:
                print 'Skip %s: %s' % (fname, e)
                continue
            for el in tree.getiterator():
                if self.options.loose_kid_support or el.get('lang', None):
                    tag = re.sub('({[^}]+})?(\w+)', '\\2', el.tag)
                    ents = []
                    if el.text: ents = [el.text.strip()]
                    if el.attrib: ents.extend(el.attrib.values())
                    for k in ents:
                        key = None
                        s = kid_expr_re.search(k)
                        if s:
                            key = s.groupdict()['texta'] or s.groupdict()['textb']
                        if key and (key not in keys) and (tag not in tags_to_ignore):
                            messages.append((tag, fname, key))
                            keys.append(key)
        fd = open(potfile, 'at+')
        for tag,fname,text in messages:
            text = normalize(text.encode('utf-8'))
            print >>fd, '#: %s:%s' % (fname, tag)
            print >>fd, 'msgid %s' % text
            print >>fd, 'msgstr ""'
            print >>fd, ''
        fd.close()
    def get_potfile_path(self):
        return os.path.join(self.locale_dir, '%s.pot' % self.domain)
    def get_locale_catalog(self, code):
        return os.path.join(self.locale_dir, code, 'LC_MESSAGES', '%s.po' % self.domain)
    def list_message_catalogs(self):
        files = []
        for name in glob.glob(self.locale_dir + '/*'):
            if os.path.isdir(name):
                fname = os.path.join(name, 'LC_MESSAGES', '%s.po' % self.domain)
                if os.path.isfile(fname):
                    files.append(fname)
        return files
    def fix_tzinfo(self, potfile):
        """
        In certain enviroments, tz info as formatted by strftime() is not utf-8.
        E.g. Windows XP with russian MUL.

        This leads to later error when a program trying to read catalog.
        """
        data = file(potfile, 'rb').read()
        def repl(m):
            "Remove tzinfo if it breaks encoding."
            tzinfo = m.group(2)
            try:
                tzinfo.decode('utf-8')
            except UnicodeDecodeError, e:
                return m.group(1) # cut tz info
            return m.group(0) # leave unchanged
        data = re.sub("(POT-Creation-Date: [\d-]+ [0-9:]+)\+([^\\\\]+)", repl, data)
        file(potfile, 'wb').write(data)

def main():
    tool = InternationalizationTool()
    tool.run()

if __name__ == '__main__':
    main()



import time

import cherrypy

import turbogears
from warnings import warn

class FeedController(turbogears.controllers.Controller):
    """ Object for generating feeds in multiple formats """

    def __init__(self, default="atom1.0"):
        self.default = default
        self.formats = ["atom1.0", "atom0.3", "rss2.0"]

    def date_to_3339(self, date):
        date = date.strftime("%Y-%m-%dT%H:%M:%SZ")
        return date

    def date_to_822(self, date):
        date = date.strftime("%a, %d %b %Y %H:%M:%S GMT")
        return date
    
    def depr_entrys(self, feed):
        if "entrys" in feed:
            warn("You should use 'entries' instead of 'entrys'",
                DeprecationWarning, 3)
            feed['entries'] = feed['entrys']
            del feed['entrys']

    def format_dates(self, feed, format):
        if format == 822:
            convert_date = self.date_to_822
        else:
            convert_date = self.date_to_3339
        if feed.has_key('updated'):
            feed["updated"] = convert_date(feed["updated"])
        self.depr_entrys(feed)
        for entry in feed['entries']:
            if entry.has_key('updated'):
                entry["updated"] = convert_date(entry["updated"])
            if entry.has_key('published'):
                entry["published"] = convert_date(entry["published"])
        return feed

    def index(self):
        raise cherrypy.HTTPRedirect(turbogears.url("%s" % self.default))
    index = turbogears.expose()(index)

    def atom1_0(self, **kwargs):
        feed = self.get_feed_data(**kwargs)
        self.format_dates(feed, 3339)
        feed["href"] = turbogears.url("/") + "atom1.0"
        self.depr_entrys(feed)
        return feed
    atom1_0 = turbogears.expose(template="turbogears.feed.atom1_0",
                    format="xml", content_type="application/atom+xml")(atom1_0)

    def atom0_3(self, **kwargs):
        feed = self.get_feed_data(**kwargs)
        self.format_dates(feed, 3339)
        feed["href"] = turbogears.url("/") + "atom0.3"
        self.depr_entrys(feed)
        return feed
    atom0_3 = turbogears.expose(template="turbogears.feed.atom0_3",
                    format="xml", content_type="application/atom+xml")(atom0_3)

    def rss2_0(self, **kwargs):
        feed = self.get_feed_data(**kwargs)
        self.format_dates(feed, 822)
        self.depr_entrys(feed)
        return feed
    rss2_0 = turbogears.expose(template="turbogears.feed.rss2_0",
                    format="xml", content_type="application/rss+xml")(rss2_0)



# Formatting configuration for locale ms_MY

languages={'ms': 'bahasa Melayu'}

countries={'BD': 'Bangladesh', 'BE': 'Belgium', 'BF': 'Burkina Faso', 'BG': 'Bulgaria', 'BA': 'Bosnia and Herzegovina', 'BB': 'Barbados', 'WF': 'Wallis and Futuna', 'BM': 'Bermuda', 'BN': 'Brunei', 'BO': 'Bolivia', 'BH': 'Bahrain', 'BI': 'Burundi', 'BJ': 'Benin', 'BT': 'Bhutan', 'JM': 'Jamaica', 'BV': 'Bouvet Island', 'BW': 'Botswana', 'WS': 'Samoa', 'BR': 'Brazil', 'BS': 'Bahamas', 'BY': 'Belarus', 'BZ': 'Belize', 'RU': 'Russia', 'RW': 'Rwanda', 'TL': 'Timor-Leste', 'RE': u'R\xe9union', 'TM': 'Turkmenistan', 'TJ': 'Tajikistan', 'RO': 'Romania', 'TK': 'Tokelau', 'GW': 'Guinea-Bissau', 'GU': 'Guam', 'GT': 'Guatemala', 'GS': 'South Georgia and South Sandwich Islands', 'GR': 'Greece', 'GQ': 'Equatorial Guinea', 'GP': 'Guadeloupe', 'JP': 'Japan', 'GY': 'Guyana', 'GF': 'French Guiana', 'GE': 'Georgia', 'GD': 'Grenada', 'GB': 'United Kingdom', 'GA': 'Gabon', 'SV': 'El Salvador', 'GN': 'Guinea', 'GM': 'Gambia', 'GL': 'Greenland', 'GI': 'Gibraltar', 'GH': 'Ghana', 'OM': 'Oman', 'TN': 'Tunisia', 'JO': 'Jordan', 'SP': 'Serbia', 'HR': 'Croatia', 'HT': 'Haiti', 'HU': 'Hungary', 'HK': 'Hong Kong S.A.R., China', 'HN': 'Honduras', 'HM': 'Heard Island and McDonald Islands', 'VE': 'Venezuela', 'PR': 'Puerto Rico', 'PS': 'Palestinian Territory', 'PW': 'Palau', 'PT': 'Portugal', 'SJ': 'Svalbard and Jan Mayen', 'PY': 'Paraguay', 'IQ': 'Iraq', 'PA': 'Panama', 'PF': 'French Polynesia', 'PG': 'Papua New Guinea', 'PE': 'Peru', 'PK': 'Pakistan', 'PH': 'Philippines', 'PN': 'Pitcairn', 'PL': 'Poland', 'PM': 'Saint Pierre and Miquelon', 'ZM': 'Zambia', 'EH': 'Western Sahara', 'EE': 'Estonia', 'EG': 'Egypt', 'ZA': 'South Africa', 'EC': 'Ecuador', 'IT': 'Italy', 'VN': 'Vietnam', 'SB': 'Solomon Islands', 'ET': 'Ethiopia', 'SO': 'Somalia', 'ZW': 'Zimbabwe', 'SA': 'Saudi Arabia', 'ES': 'Spain', 'ER': 'Eritrea', 'MD': 'Moldova', 'MG': 'Madagascar', 'MA': 'Morocco', 'MC': 'Monaco', 'UZ': 'Uzbekistan', 'MM': 'Myanmar', 'ML': 'Mali', 'MO': 'Macao S.A.R. China', 'MN': 'Mongolia', 'MH': 'Marshall Islands', 'MK': 'Macedonia', 'MU': 'Mauritius', 'MT': 'Malta', 'MW': 'Malawi', 'MV': 'Maldives', 'MQ': 'Martinique', 'MP': 'Northern Mariana Islands', 'MS': 'Montserrat', 'MR': 'Mauritania', 'UG': 'Uganda', 'MY': 'Malaysia', 'MX': 'Mexico', 'IL': 'Israel', 'FR': 'France', 'IO': 'British Indian Ocean Territory', 'SH': 'Saint Helena', 'FI': 'Finland', 'FJ': 'Fiji', 'FK': 'Falkland Islands', 'FM': 'Micronesia', 'FO': 'Faroe Islands', 'NI': 'Nicaragua', 'NL': 'Netherlands', 'NO': 'Norway', 'NA': 'Namibia', 'VU': 'Vanuatu', 'NC': 'New Caledonia', 'NE': 'Niger', 'NF': 'Norfolk Island', 'NG': 'Nigeria', 'NZ': 'New Zealand', 'NP': 'Nepal', 'NR': 'Nauru', 'NU': 'Niue', 'CK': 'Cook Islands', 'CI': u"C\xf4te d'Ivoire", 'CH': 'Switzerland', 'CO': 'Colombia', 'CN': 'China', 'CM': 'Cameroon', 'CL': 'Chile', 'CC': 'Cocos Islands', 'CA': 'Canada', 'CG': 'Congo', 'CF': 'Central African Republic', 'CD': 'Democratic Republic of the Congo', 'CZ': 'Czech Republic', 'CY': 'Cyprus', 'CX': 'Christmas Island', 'CR': 'Costa Rica', 'Fallback': 'en', 'CV': 'Cape Verde', 'CU': 'Cuba', 'SZ': 'Swaziland', 'SY': 'Syria', 'KG': 'Kyrgyzstan', 'KE': 'Kenya', 'SR': 'Suriname', 'KI': 'Kiribati', 'KH': 'Cambodia', 'KN': 'Saint Kitts and Nevis', 'KM': 'Comoros', 'ST': 'Sao Tome and Principe', 'SK': 'Slovakia', 'KR': 'South Korea', 'SI': 'Slovenia', 'KP': 'North Korea', 'KW': 'Kuwait', 'SN': 'Senegal', 'SM': 'San Marino', 'SL': 'Sierra Leone', 'SC': 'Seychelles', 'KZ': 'Kazakhstan', 'KY': 'Cayman Islands', 'SG': 'Singapore', 'SE': 'Sweden', 'SD': 'Sudan', 'DO': 'Dominican Republic', 'DM': 'Dominica', 'DJ': 'Djibouti', 'DK': 'Denmark', 'VG': 'British Virgin Islands', 'DE': 'Germany', 'YE': 'Yemen', 'DZ': 'Algeria', 'US': 'United States', 'UY': 'Uruguay', 'YU': 'Yugoslavia', 'YT': 'Mayotte', 'UM': 'United States Minor Outlying Islands', 'LB': 'Lebanon', 'LC': 'Saint Lucia', 'LA': 'Laos', 'TV': 'Tuvalu', 'TW': 'Taiwan', 'TT': 'Trinidad and Tobago', 'TR': 'Turkey', 'LK': 'Sri Lanka', 'LI': 'Liechtenstein', 'LV': 'Latvia', 'TO': 'Tonga', 'LT': 'Lithuania', 'LU': 'Luxembourg', 'LR': 'Liberia', 'LS': 'Lesotho', 'TH': 'Thailand', 'TF': 'French Southern Territories', 'TG': 'Togo', 'TD': 'Chad', 'TC': 'Turks and Caicos Islands', 'LY': 'Libya', 'VA': 'Vatican', 'VC': 'Saint Vincent and the Grenadines', 'AE': 'United Arab Emirates', 'AD': 'Andorra', 'AG': 'Antigua and Barbuda', 'AF': 'Afghanistan', 'AI': 'Anguilla', 'VI': 'U.S. Virgin Islands', 'IS': 'Iceland', 'IR': 'Iran', 'AM': 'Armenia', 'AL': 'Albania', 'AO': 'Angola', 'AN': 'Netherlands Antilles', 'AQ': 'Antarctica', 'AS': 'American Samoa', 'AR': 'Argentina', 'AU': 'Australia', 'AT': 'Austria', 'AW': 'Aruba', 'IN': 'India', 'TZ': 'Tanzania', 'AZ': 'Azerbaijan', 'IE': 'Ireland', 'ID': 'Indonesia', 'UA': 'Ukraine', 'QA': 'Qatar', 'MZ': 'Mozambique'}

months=['Januari', 'Februari', 'Mac', 'April', 'Mei', 'Jun', 'Julai', 'Ogos', 'September', 'Oktober', 'November', 'Disember']

abbrMonths=['Jan', 'Feb', 'Mac', 'Apr', 'Mei', 'Jun', 'Jul', 'Ogos', 'Sept', 'Okt', 'Nov', 'Dis']

days=['Isnin', 'Selasa', 'Rabu', 'Khamis', 'Jumaat', 'Sabtu', 'Ahad']

abbrDays=['Isn', 'Sel', 'Rab', 'Kha', 'Jum', 'Sab', 'Aha']

dateFormats={'medium': '%d %%(monthname)s %Y', 'full': '%%(dayname)s %d %%(abbrmonthname)s %Y', 'long': '%d %%(monthname)s %Y', 'short': '%d/%m/%Y'}

numericSymbols={'group': ',', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': '.', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}


import os
from cherrypy import request
from turbogears.i18n import get_locale
from turbogears import startup, config
from turbogears.util import parse_http_accept_header
from turbogears.i18n.utils import lang_in_gettext_format
from turbogears.widgets.base import JSLink, CoreWD, RenderOnlyWD, static

__all__ = ["CalendarLangFileLink", "LocalizableJSLink"]

class CalendarLangFileLink(JSLink):
    """Links to proper calendar.js language file depending on HTTP info."""
    def update_params(self, d):
        accept_language = parse_http_accept_header(            
            request.headers.get("Accept-Language")
        ) or ['']
        accept_charset = parse_http_accept_header(
            request.headers.get("Accept-Charset")
        ) or ['']
        base_dir = config.get("static_filter.dir", 
                              path="/tg_widgets/%s" % self.mod)
        def find_link():
            for name_pattern in self.name:
                for lang in accept_language:
                    for charset in accept_charset:
                        params = dict()
                        params["lang"] = lang
                        params["charset"] = charset
                        params["gettext_lang"] = lang_in_gettext_format(lang)
                        params["gettext_charset"] = charset.upper()
                        params["custom_lang"] = self.custom_lang(lang)
                        params["custom_charset"] = self.custom_charset(charset)
                        name = name_pattern % params
                        if os.path.exists(base_dir + '/' + name):
                            link = "/%stg_widgets/%s/%s" % (startup.webpath,
                                                            self.mod,
                                                            name)
                            return link
        d["link"] = find_link()

    def custom_lang(self, lang):
        return None

    def custom_charset(self, charset):
        return None

class LocalizableJSLink(JSLink):

    """
    Provides a simple way to include language-specific data in your
    Javascript code.

    Language file to use is determined from the user's locale or from the 'language'
    parameter. If there is no language file for the language (determined via
    'supported_languages' parameter) than 'default_language' is used.
    """

    default_language = 'en'
    supported_languages = ['en']
    params = ['default_language', 'language', 'supported_languages']
    params_doc = {
        'language' : 'language code to use ' \
            '(overrides user locale setting which is the default)',
        'default_language' : 'language code to use ' \
                'if specified language is not supported',
        'supported_languages' : 'list of supported language codes ' \
                ' (which means corresponding language files exist)',
    }

    def update_params(self, d):
        super(LocalizableJSLink, self).update_params(d)
        language = d.get('language') or get_locale()
        if language not in self.supported_languages:
            language = self.default_language
        d['link'] = d['link'] % {'lang':language}

class LocalizableJSLinkDesc(CoreWD, RenderOnlyWD):
    name = "Localizable JS Link"
    for_widget = LocalizableJSLink("turbogears", "js/yourscript-%(lang)s.js")



import cherrypy
import socket
import struct
import threading
import types
import turbogears
from turbogears.identity.exceptions import *
from turbogears.identity import *
from turbogears.decorator import weak_signature_decorator

class Predicate(object):
    '''
    Generic base class for testing true or false for a condition.
    '''
    def eval_with_object( self, obj, errors=None ):
        '''
        Determine whether the predicate is True or False for the given object.
        '''
        raise NotImplementedError

    def append_error_message( self, errors=None ):
        if errors is None:
            return
        errors.append( self.error_message % self.__dict__ )


class CompoundPredicate(Predicate):
    '''
    A predicate composed of other predicates.
    '''
    def __init__(self, *predicates):
        self.predicates= predicates


class All(CompoundPredicate):
    '''
    A compound predicate that evaluates to true only if all sub-predicates
    evaluate to true for the given input.
    '''
    def eval_with_object( self, obj, errors=None ):
        '''
        Return true if all sub-predicates evaluate to true.
        '''
        for p in self.predicates:
            if not p.eval_with_object( obj, errors ):
                return False
        return True


class Any(CompoundPredicate):
    '''
    A compound predicate that evaluates to true if any one of its sub-predicates
    evaluates to true.
    '''
    error_message= "No predicates were able to grant access"

    def eval_with_object( self, obj, errors=None ):
        '''
        Return true if any sub-predicate evaluates to true.
        '''
        for p in self.predicates:
            if p.eval_with_object( obj, None ):
                return True
        self.append_error_message( errors )
        return False


class IdentityPredicateHelper(object):
    '''
    A mix-in helper class for Identity Predicates.
    '''
    def __nonzero__(self):
        return self.eval_with_object( current )


class in_group(Predicate, IdentityPredicateHelper):
    '''
    Predicate for requiring a group.
    '''
    error_message= "Not member of group: %(group_name)s"

    def __init__(self, group_name):
        self.group_name= group_name

    def eval_with_object( self, identity, errors=None ):
        if self.group_name in identity.groups:
            return True
        self.append_error_message( errors )
        return False


class in_all_groups(All, IdentityPredicateHelper):
    '''
    Predicate for requiring membership in a number of groups.
    '''
    def __init__(self, *groups):
        group_predicates= [in_group(g) for g in groups]
        super(in_all_groups,self).__init__( *group_predicates )


class in_any_group(Any, IdentityPredicateHelper):
    '''
    Predicate for requiring membership in at least one group
    '''
    error_message= "Not member of any group: %(group_list)s"

    def __init__(self, *groups):
        self.group_list= ", ".join(groups)
        group_predicates= [in_group(g) for g in groups]
        super(in_any_group,self).__init__( *group_predicates )


class not_anonymous(Predicate, IdentityPredicateHelper):
    '''
    Predicate for checking whether current visitor is anonymous.
    '''
    error_message= "Anonymous access denied"

    def eval_with_object( self, identity, errors=None ):
        if current.anonymous:
            self.append_error_message( errors )
            return False
        return True


class has_permission(Predicate, IdentityPredicateHelper):
    '''
    Predicate for checking whether the visitor has a particular permission.
    '''
    error_message= "Permission denied: %(permission_name)s"

    def __init__(self, permission_name):
        self.permission_name= permission_name

    def eval_with_object(self, identity, errors=None):
        '''
        Determine whether the visitor has the specified permission.
        '''
        if self.permission_name in identity.permissions:
            return True

        self.append_error_message( errors )
        return False


class has_all_permissions(All, IdentityPredicateHelper):
    '''
    Predicate for checking whether the visitor has all permissions.
    '''
    def __init__(self, *permissions):
        permission_predicates= [has_permission(p) for p in permissions]
        super(has_all_permissions,self).__init__( *permission_predicates )


class has_any_permission(Any, IdentityPredicateHelper):
    '''
    Predicate for checking whether the visitor has at least one permission.
    '''
    error_message= "No matching permissions: %(permission_list)s"

    def __init__(self, *permissions):
        self.permission_list= ", ".join( permissions )
        permission_predicates= [has_permission(p) for p in permissions]
        super(has_any_permission,self).__init__( *permission_predicates )


def _remoteHost():
    try:
        ips= cherrypy.request.headers.get( "X-Forwarded-For",
                                             cherrypy.request.remote_host )
        return ips.split(",")[-1].strip()
    except:
        return ""


def _match_ip(cidr, ip):
    if not '/' in cidr:
        return cidr == ip
    else:
        try:
            b,m = cidr.split('/')
            shift = 32 - int(m)
            a1 = struct.unpack('!L', socket.inet_aton(b))[0] >> shift
            a2 = struct.unpack('!L', socket.inet_aton(ip))[0] >> shift
            return a1 == a2
        except:
            return False


class from_host(Predicate, IdentityPredicateHelper):
    '''
    Predicate for checking whether the visitor's host is an allowed host.
    Note: We never want to announce what the list of allowed hosts is, because
    it is way too easy to spoof an IP address in a TCP/IP packet.
    '''
    error_message= "Access from this host is not permitted."

    def __init__(self, host):
        self.host= host

    def eval_with_object( self, obj, errors=None ):
        '''
        Match the visitor's host against the criteria.
        '''
        ip = _remoteHost()
        if _match_ip( self.host, ip ):
            return True
        self.append_error_message( errors )
        return False


class from_any_host(Any, IdentityPredicateHelper):
    '''
    Predicate for checking whether the visitor's host is one of a number of
    permitted hosts.
    '''
    error_message= "Access from this host is not permitted."

    def __init__(self, hosts):
        host_predicates= [from_host(h) for h in hosts]
        super(from_any_host,self).__init__( *host_predicates )


def require(predicate, obj=None):
    '''
    Function decorator that checks whether the current user is a member of the
    groups specified and has the permissions required.
    '''
    def entangle(fn):
        def require(func, self, *args, **kwargs):
            try:
                errors= []
                if predicate is None or \
                   predicate.eval_with_object(current, errors):
                   return fn(self, *args, **kwargs)
            except IdentityException, e:
                errors= [str(e)]

            raise IdentityFailure(errors)

        fn._require= predicate
        return require
    return weak_signature_decorator(entangle)

def _secureResourceDecorator( fn ):
    '''
    A decorator function used by the SecureResourceMeta metaclass.
    '''
    def _wrapper( self, *args, **kwargs ):
        predicate= getattr( self.__class__, "require", None )

        try:
            errors= []
            if predicate is None or \
               predicate.eval_with_object( current, errors ):
               return fn( self, *args, **kwargs )
        except IdentityException, e:
            errors= [str(e)]

        raise IdentityFailure( errors )

    try:
        _wrapper.func_name= fn.func_name
    except TypeError:
        pass
    if hasattr( fn, "exposed" ):
        _wrapper.exposed= fn.exposed
    return _wrapper


def _check_method( obj, fn, predicate ):
    def _wrapper( *args, **kw ):
        errors= []
        if predicate.eval_with_object( current, errors ):
            return fn( *args, **kw )
        else:
            raise IdentityFailure( errors )
    _wrapper.exposed= True
    return _wrapper


class SecureResource(object):

    def __getattribute__( self, name ):
        from turbogears import controllers
        if name[:3]=="_cp" or name in ["require"]:
            return object.__getattribute__(self,name)
        try:
            value= object.__getattribute__(self,name)
            try:
                predicate= object.__getattribute__(self,"require")
            except AttributeError:
                predicate= turbogears.config.get( "identity.require", None )

            if predicate is None:
                raise AttributeError("SecureResource requires a require "
                                     "attribute either on the controller class "
                                     "itself or in the config file")

            errors= []
            if (isinstance( value, types.MethodType ) and
                hasattr( value, "exposed" )):
                return _check_method( self, value, predicate )
            if isinstance( value, controllers.Controller ):
                return SecureObject( value, predicate )
            # Some other property
            return value
        except IdentityException, e:
            errors= [str(e)]

        raise IdentityFailure( errors )

class SecureObject(object):
    def __init__( self, obj, require, exclude=[]):
        self._exclude = exclude
        self._object= obj
        self._require= require

    def __getattribute__( self, name ):
        from turbogears import controllers
        if name[:3]=="_cp" or name in ["_object","_require","_exclude"]:
            return object.__getattribute__(self,name)
        try:
            obj= object.__getattribute__( self, "_object" )
            value= getattr( obj, name )

            errors= []
            predicate= object.__getattribute__(self,"_require")
            if name in object.__getattribute__(self, "_exclude"):
                return value
            if (isinstance( value, types.MethodType ) and
                hasattr( value, "exposed" )):
                return _check_method( obj, value, predicate )
            if isinstance( value, controllers.Controller ):
                return SecureObject( value, predicate )
            # Some other property
            return value
        except IdentityException, e:
            errors= [str(e)]

        raise IdentityFailure( errors )



#!/usr/bin/env python

import os
import sys
import subprocess
import re

externals = {"cherrypy" : "http://svn.cherrypy.org/tags/cherrypy-2.2.1/",
"sqlobject" : "http://svn.colorstudy.com/SQLObject/branches/0.7-bugfix/",
"mochikit" : "http://svn.mochikit.com/mochikit/tags/MochiKit-1.3.1/",
"kid" : "svn://svn.kid-templating.org/tags/0.9.3/",
"formencode" : "http://svn.colorstudy.com/FormEncode/tags/0.5.1/",
"paste" : "http://svn.pythonpaste.org/Paste/tags/0.5/",
"pastescript" : "http://svn.pythonpaste.org/Paste/Script/tags/0.5.1/",
}

urlline = re.compile("^URL: (.*)")

def update(proj):
    os.chdir(proj)
    svninfo = subprocess.Popen(["svn", "info"], stdout=subprocess.PIPE)
    out = svninfo.stdout
    line = out.readline()
    while line:
        line = out.readline()
        url = urlline.match(line)
        if url:
            url = url.group(1).strip()
            if url != externals[proj] and url != externals[proj][:-1]:
                print "Switching project %s to %s" % (proj, externals[proj])
                svnswitch = subprocess.call(["svn", "switch", externals[proj], "."])
            else:
                print "Updating project %s" % (proj)
                svnupdate = subprocess.call(["svn", "update"])
            break
    if not url:
        print "Error! Unable to find URL for project %s"
    os.chdir("..")

def checkout(proj):
    print "Checking out %s" % (proj)
    svncheckout = subprocess.call(["svn", "checkout", externals[proj], proj])

def run():
    if not os.path.exists("externals.py"):
        print "You must run this script from the thirdparty directory."
        sys.exit(1)
        
    for proj in externals:
        if os.path.exists(proj):
            update(proj)
        else:
            checkout(proj)

if __name__ == "__main__":
    run()


import os
import sys
import re
import htmlentitydefs
from inspect import getargspec, getargvalues
from itertools import izip, islice, chain, imap
from operator import isSequenceType

import pkg_resources
import setuptools

from cherrypy import request
from turbogears.decorator import decorator
from turbogears import config


# This Enum implementation is from the Python Cookbook and is
# written by Zoran Isailovski:
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/413486

def Enum(*names):
    ##assert names, "Empty enums are not supported" # <- Don't like empty enums? Uncomment!

    class EnumClass(object):
        __slots__ = names
        def __iter__(self):          return iter(constants)
        def __len__(self):            return len(constants)
        def __getitem__(self, i):  return constants[i]
        def __repr__(self):          return 'Enum' + str(names)
        def __str__(self):            return 'enum ' + str(constants)

    enumType = EnumClass()

    class EnumValue(object):
        __slots__ = ('__value')
        def __init__(self, value): self.__value = value
        Value = property(lambda self: self.__value)
        EnumType = property(lambda self: enumType)
        def __hash__(self):          return hash(self.__value)
        def __cmp__(self, other):
            # C fans might want to remove the following assertion
            # to make all enums comparable by ordinal value {;))
            assert self.EnumType is other.EnumType, "Only values from the same enum are comparable"
            return cmp(self.__value, other.__value)
        def __invert__(self):        return constants[maximum - self.__value]
        def __nonzero__(self):      return bool(self.__value)
        def __repr__(self):          return str(names[self.__value])

    maximum = len(names) - 1
    constants = [None] * len(names)
    for i, each in enumerate(names):
        val = EnumValue(i)
        setattr(EnumClass, each, val)
        constants[i] = val
    constants = tuple(constants)
    return enumType

class setlike(list):
    """Set preserving item order."""

    def add(self, item):
        if item not in self:
            self.append(item)

    def add_all(self, iterable):
        for item in iterable:
            self.add(item)

def get_project_meta(name):
    for dirname in os.listdir("./"):
        if dirname.lower().endswith("egg-info"):
            fname = os.path.join(dirname, name)
            return fname

def get_project_config():
    """Tries to select appropriate project configuration file."""

    config = None
    if os.path.exists("setup.py"):
        config = "dev.cfg"
    else:
        config = "prod.cfg"
    return config

def load_project_config(configfile=None):
    """Tries to update the config, loading project settings from the config
    file specified.  If config is C{None} uses L{get_project_config} to locate
    one.
    """
    if configfile is None:
        configfile = get_project_config()
    if not os.path.isfile(configfile):
        print 'config file %s not found or is not a file.' % os.path.abspath(configfile)
        sys.exit()
    package = get_package_name()
    config.update_config(configfile=configfile,
        modulename = package + ".config")

def get_package_name():
    """Try to find out the package name of the current directory."""
    if "--egg" in sys.argv:
        projectname = sys.argv[sys.argv.index("--egg")+1]
        egg = pkg_resources.get_distribution(projectname)
        package = list(egg._get_metadata("top_level.txt"))[0]
        return package
    fname = get_project_meta('top_level.txt')
    if fname:
        return open(fname).readline()[:-1]

def get_project_name():
    pkg_info = get_project_meta('PKG-INFO')
    if pkg_info:
        name = list(open(pkg_info))[1][6:-1]
        return name

def get_model():
    package_name = get_package_name()
    if not package_name:
        return None
    package = __import__(package_name, {}, {}, ["model"])
    if hasattr(package, "model"):
        return package.model


def ensure_sequence(obj):
    """Construct a sequence from object."""
    if obj is None:
        return []
    elif isSequenceType(obj):
        return obj
    else:
        return [obj]

def to_kw(func, args, kw, start=0):
    """Convert all applicable arguments to keyword arguments."""
    argnames, defaults = getargspec(func)[::3]
    defaults = ensure_sequence(defaults)
    kv_pairs = izip(islice(argnames, start, len(argnames) - len(defaults)), args)
    for k, v in kv_pairs:
        kw[k] = v
    return args[len(argnames)-len(defaults)-start:], kw

def from_kw(func, args, kw, start=0):
    """Extract named positional arguments from keyword arguments."""
    argnames, defaults = getargspec(func)[::3]
    defaults = ensure_sequence(defaults)
    newargs = [kw.pop(name) for name in islice(argnames, start,
               len(argnames) - len(defaults)) if name in kw]
    newargs.extend(args)
    return newargs, kw

def adapt_call(func, args, kw, start=0):
    """Remove excess arguments."""
    argnames, varargs, kwargs, defaults = getargspec(func)
    defaults = ensure_sequence(defaults)
    del argnames[:start]
    if kwargs in (None, "_decorator__kwargs"):
        remove_keys(kw, [key for key in kw.iterkeys() if key not in argnames])
    if varargs in (None, "_decorator__varargs"):
        args = args[:len(argnames) - len(defaults)]
    else:
        pivot = len(argnames) - len(defaults)
        args = tuple(chain(islice(args, pivot), imap(kw.pop, islice(
                        argnames, pivot, None)), islice(args, pivot, None)))
    return args, kw

def call_on_stack(func_name, kw, start=0):
    """Check if a call to function matching pattern is on stack. """
    try:
        frame = sys._getframe(start+1)
    except ValueError:
        return False
    while frame.f_back:
        frame = frame.f_back
        if frame.f_code.co_name == func_name:
            args = getargvalues(frame)[3]
            for key in kw.iterkeys():
                try:
                    if kw[key] != args[key]:
                        continue
                except KeyError, TypeError:
                    continue
            if key or not args:
                return True
    return False

def arg_index(func, argname):
    """Find index of argument as declared for given function."""
    argnames = getargspec(func)[0]
    if has_arg(func, argname):
        return argnames.index(argname)
    else:
        return None

def has_arg(func, argname):
    """Check whether function has argument."""
    return argname in getargspec(func)[0]

def inject_arg(func, argname, argval, args, kw, start=0):
    """Insert argument into call."""
    argnames, defaults = getargspec(func)[::3]
    defaults = ensure_sequence(defaults)
    pos = arg_index(func, argname)
    if pos is None or pos > len(argnames) - len(defaults) - 1:
        kw[argname] = argval
    else:
        pos -= start
        args = tuple(chain(islice(args, pos), (argval,),
                           islice(args, pos, None)))
    return args, kw

def inject_args(func, injections, args, kw, start=0):
    """Insert arguments into call."""
    for argname, argval in injections.iteritems():
        args, kw = inject_arg(func, argname, argval, args, kw, start)
    return args, kw

def inject_call(func, injections, *args, **kw):
    """Insert arguments and call."""
    args, kw = inject_args(func, injections, args, kw)
    return func(*args, **kw)

def bind_args(**add):
    """Call with arguments set to a predefined value."""
    def entagle(func):
        return lambda func, *args, **kw: inject_call(func, add, *args, **kw)

    def make_decorator(func):
        argnames, varargs, kwargs, defaults = getargspec(func)
        defaults = list(ensure_sequence(defaults))
        defaults = [d for d in defaults if
                    argnames[-len(defaults) + defaults.index(d)] not in add]
        argnames = [arg for arg in argnames if arg not in add]
        return decorator(entagle, (argnames, varargs, kwargs, defaults))(func)

    return make_decorator

def remove_keys(dict_, seq):
    """Gracefully remove keys from dict."""
    for key in seq:
        dict_.pop(key, None)
    return dict_

def recursive_update(to_dict, from_dict):
    """Recursively update all dicts in to_dict with values from from_dict."""
    # probably slow as hell :( should be optimized somehow...
    for k, v in from_dict.iteritems():
        if isinstance(v, dict) and isinstance(to_dict[k], dict):
            recursive_update(to_dict[k], v)
        else:
            to_dict[k] = v
    return to_dict

def combine_contexts(frames=None, depth=None):
    """Combine contexts (globals, locals) of frames."""
    locals_ = {}
    globals_ = {}
    if frames is None:
        frames = []
    if depth is not None:
        frames.extend([sys._getframe(d+1) for d in depth])
    for frame in frames:
        locals_.update(frame.f_locals)
        globals_.update(frame.f_globals)
    return locals_, globals_

def request_available():
    """Check if cherrypy.request is available."""
    try:
        setattr(request, "tg_dumb_attribute", True)
        return True
    except AttributeError:
        return False


def flatten_sequence(seq):
    """Flatten sequence."""
    for item in seq:
        if isSequenceType(item) and not isinstance(item, basestring):
            for item in flatten_sequence(item):
                yield item
        else:
            yield item


def load_class(dottedpath):
    '''
    Loads a class from a module in dotted-path notation.
    Eg: load_class("package.module.class").

    Based on recipe 16.3 from "Python Cookbook, 2ed., by Alex Martelli,
    Anna Martelli Ravenscroft, and David Ascher (O'Reilly Media, 2005)
    0-596-00797-3"
    '''
    splitted_path = dottedpath.split('.')
    modulename = '.'.join(splitted_path[:-1])
    classname = splitted_path[-1]
    try:
        module = __import__(modulename, globals(), locals(), [classname])
    except ImportError:
        return None
    return getattr(module, classname)

class Bunch(dict):
    __setattr__ = dict.__setitem__

    def __delattr__(self, name):
        try:
            del self[name]
        except KeyError:
            raise AttributeError(name)

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError(name)

#XXX: Should issue Deprecation warning?
DictObj = Bunch
DictWrapper = Bunch

def parse_http_accept_header(accept):
    items = []
    if accept is None:
        return items
    for item in accept.split(","):
        pos = item.find(";q=")
        order = 1
        if pos > -1:
            order = float(item[pos+3:].strip())
            item = item[:pos].strip()
        items.append((item, order))
    items.sort(lambda i1, i2: cmp(i2[1], i1[1]))
    return [i[0] for i in items]

def to_unicode(value):
    """
    Converts encoded string to unicode string.

    Uses get_template_encoding_default() to guess source string encoding.
    Handles turbogears.i18n.lazystring correctly.
    """
    if isinstance(value, str):
        # try to make sure we won't get UnicodeDecodeError from the template
        # by converting all encoded strings to Unicode strings
        try:
            value = unicode(value)
        except UnicodeDecodeError:
            try:
                value = unicode(value, get_template_encoding_default())
            except UnicodeDecodeError:
                # fail early
                raise ValueError("Non-unicode string: %r" % value)
    return value

def to_utf8(value):
    """Converts a unicode string to utf-8 encoded plain string.

    Handles turbogears.i18n.lazystring correctly. Does nothing to already encoded string.
    """
    if isinstance(value, str):
        pass
    elif hasattr(value, '__unicode__'):
        value = unicode(value)
    if isinstance(value, unicode):
        value = value.encode('utf-8')
    return value

def get_template_encoding_default(engine_name=None):
    """Returns default encoding for template files (Kid, Genshi, etc.)."""
    if engine_name is None:
        engine_name = config.get('tg.defaultview', 'kid')
    return config.get('%s.encoding' % engine_name, 'utf-8')

def find_precision(value):
    """
    Find precision of some arbitrary value.  The main intention for this function
    is to use it together with turbogears.i18n.format.format_decimal() where one
    has to inform the precision wanted.  So, use it like this:

    format_decimal(some_number, find_precision(some_number))
    """
    decimals = ''
    try:
        stub, decimals = str(value).split('.')
    except ValueError:
        pass
    return len(decimals)

def copy_if_mutable(value, feedback=False):
    if isinstance(value, dict):
        mutable = True
        value = value.copy()
    elif isinstance(value, list):
        mutable = True
        value = value[:]
    else:
        mutable = False
    if feedback:
        return (value, mutable)
    else:
        return value

def fixentities(htmltext):
    # replace HTML character entities with numerical references
    # note: this won't handle CDATA sections properly
    def repl(m):
        entity = htmlentitydefs.entitydefs.get(m.group(1).lower())
        if not entity:
            return m.group(0)
        elif len(entity) == 1:
            if entity in "&<>'\"":
                return m.group(0)
            return "&#%d;" % ord(entity)
        else:
            return entity
    return re.sub("&(\w+);?", repl, htmltext)


__all__ = ["Enum", "setlike",
           "get_package_name", "get_model", "load_project_config",
           "url", "ensure_sequence", "has_arg",
           "DictWrapper", "DictObj", "to_kw", "from_kw", "adapt_call",
           "call_on_stack", "remove_keys",
           "arg_index", "inject_arg", "inject_args", "bind_args",
           "recursive_update", "combine_contexts", "request_available",
           "flatten_sequence", "load_class", "Bunch",
           "parse_http_accept_header", 
           "to_unicode", "to_utf8", "get_template_encoding_default",
           "find_precision", "copy_if_mutable"]



"""Classes and methods for TurboGears controllers."""
import logging
import re
import urllib
import types
from itertools import izip
import cherrypy
from dispatch import generic, strategy, functions
import turbogears.util as tg_util
import turbogears 
from inspect import isclass
from turbogears import view, database, errorhandling, config
from turbogears.decorator import weak_signature_decorator
from turbogears.validators import Invalid
from turbogears.errorhandling import error_handler, exception_handler

log = logging.getLogger("turbogears.controllers")

unicodechars = re.compile(r"([^\x00-\x7F])")

if config.get("session_filter.on",None) == True:
    if config.get("session_filter.storage_type",None) == "PostgreSQL":
        import psycopg2 
        config.update({'session_filter.get_db':psycopg2.connect(psycopg2.get('sessions.postgres.dsn'))})
    # support for mysql/sqlite/etc here

def _process_output(output, template, format, content_type, mapping, fragment=False):
    """Produces final output form from the data returned from a
    controller method.

    @param tg_format: format of desired output (html or json)
    @param output: the output returned by the controller
    @param template: HTML template to use
    """
    if isinstance(output, dict):
        from turbogears.widgets import js_location

        css = tg_util.setlike()
        js = dict(izip(js_location, iter(tg_util.setlike, None)))
        include_widgets = {}
        include_widgets_lst = config.get("tg.include_widgets", [])

        if config.get("tg.mochikit_all", False):
            include_widgets_lst.insert(0, 'turbogears.mochikit')
            
        for i in include_widgets_lst:
            widget = tg_util.load_class(i)  
            if isclass(widget):
                widget = widget()
            include_widgets["tg_%s" % i.split(".")[-1]] = widget
            for script in widget.retrieve_javascript():
                if hasattr(script, "location"):
                    js[script.location].add(script)
                else:
                    js[js_location.head].add(script)
            css.add_all(widget.retrieve_css())

        for value in output.itervalues():
            if hasattr(value, "retrieve_css"):
                retrieve = getattr(value, "retrieve_css")
                if callable(retrieve):
                    css.add_all(value.retrieve_css())
            if hasattr(value, "retrieve_javascript"):
                retrieve = getattr(value, "retrieve_javascript")
                if callable(retrieve):
                    for script in value.retrieve_javascript():
                        if hasattr(script, "location"):
                            js[script.location].add(script)
                        else:
                            js[js_location.head].add(script)
        output.update(include_widgets)
        output["tg_css"] = css
        #output.update([("tg_js_%s" % str(l), js[l]) for l in js_location])
        for l in iter(js_location):
            output["tg_js_%s" % str(l)] = js[l]

        tg_flash = _get_flash() 
        if not tg_flash == None:
            output["tg_flash"] = tg_flash 
        output = view.render(output, template=template, format=format,
                    mapping=mapping, content_type=content_type,
                    fragment=fragment)
    else:
        if content_type:
            cherrypy.response.headers["Content-Type"] = content_type

    # fix the Safari XMLHttpRequest encoding problem
    try:
        contentType = cherrypy.response.headers["Content-Type"]
        ua = cherrypy.request.headers["User-Agent"]
    except KeyError:
        return output
    if not contentType.startswith("text/"):
        return output
    ua = view.UserAgent(ua)
    enc = tg_util.get_template_encoding_default()
    if ua.browser == "safari":
        if isinstance(output, str):
            output = output.decode(enc)
        output = unicodechars.sub(
            lambda m: "&#x%x;" % ord(m.group(1)), output).encode("ascii")
    if isinstance(output, unicode):
        output = output.encode(enc)
    return output

class BadFormatError(Exception):
    """Output-format exception."""

def validate(form=None, validators=None,
             failsafe_schema=errorhandling.FailsafeSchema.none,
             failsafe_values=None, state_factory=None):
    """Validate input.

    @param form form to validate input from
    @param validators individual validators to use for parameters
    @param failsafe_schema fail-safe schema
    @param failsafe_values replacements for erroneous inputs
    @param state_factory callable which returns the initial state instance for
           validation
    """
    def entangle(func):
        recursion_guard = dict(func=func)
        if callable(form) and not hasattr(form, "validate"):
            init_form = lambda self: form(self)
        else:
            init_form = lambda self: form

        def validate(func, *args, **kw):
            if tg_util.call_on_stack("validate", recursion_guard, 4):
                return func(*args, **kw)
            form = init_form(args and args[0] or kw["self"])
            args, kw = tg_util.to_kw(func, args, kw)

            errors = {}
            if state_factory is not None:
                state = state_factory()
            else:
                state = None

            if form:
                value = kw.copy()
                try:
                    kw.update(form.validate(value, state))
                except Invalid, e:
                    errors = e.unpack_errors()
                    cherrypy.request.validation_exception = e
                cherrypy.request.validated_form = form

            if validators:
                if isinstance(validators, dict):
                    for field, validator in validators.iteritems():
                        try:
                            kw[field] = validator.to_python(
                                kw.get(field, None), state
                                )
                        except Invalid, error:
                            errors[field] = error
                else:
                    try:
                        value = kw.copy()
                        kw.update(validators.to_python(value, state))
                    except Invalid, e:
                        errors = e.unpack_errors()
                        cherrypy.request.validation_exception = e
            cherrypy.request.validation_errors = errors
            cherrypy.request.input_values = kw.copy()
            cherrypy.request.validation_state = state

            if errors:
                kw = errorhandling.dispatch_failsafe(failsafe_schema,
                                            failsafe_values, errors, func, kw)
            args, kw = tg_util.from_kw(func, args, kw)
            return errorhandling.run_with_errors(errors, func, *args, **kw)

        return validate
    return weak_signature_decorator(entangle)

class CustomDispatch(functions.GenericFunction):

    def combine(self,cases):
        strict = [strategy.ordered_signatures,strategy.safe_methods]
        cases = strategy.separate_qualifiers(
            cases,
            primary = strict,
        )
        primary = strategy.method_chain(cases.get('primary',[]))
        if type(primary) != types.FunctionType:
            for i in primary:
                for y in i:
                    return y[1]
        return primary

def _add_rule(_expose, found_default, as_format, accept_format, template,
              rulefunc):
    if as_format == "default":
        if found_default:
            colon = template.find(":")
            if colon == -1:
                as_format = template
            else:
                as_format = template[:colon]
        else:
            found_default = True
    ruleparts = []
    ruleparts.append('kw.get("tg_format", "default") == "%s"'
                % as_format)
    if accept_format:
        ruleparts.append('(accept == "%s" and kw.get("tg_format", '
                '"default") == "default")' % accept_format)
    rule = " or ".join(ruleparts)
    log.debug("Generated rule %s", rule)
    _expose.when(rule)(rulefunc)

    return found_default

def _build_rules(func):
    def _expose(func, accept, allow_json, *args, **kw):
        pass
    _expose = generic(CustomDispatch)(_expose)

    if func._allow_json:
        log.debug("Adding allow_json rule: "
            'allow_json and '
            '(kw.get("tg_format", None) == "json" or accept '
            '=="text/javascript")')
        _expose.when('allow_json '
            'and (kw.get("tg_format", None) == "json" or accept'
            ' =="text/javascript")')(
            lambda _func, accept, allow_json,
                *args, **kw: _execute_func(
                    _func, "json", None, None, None, False, args, kw))

    found_default = False
    for ruleinfo in func._ruleinfo:
        found_default = _add_rule(_expose, found_default, **ruleinfo)

    func._expose = _expose

def expose(template=None, validators=None, allow_json=None, html=None,
           format=None, content_type=None, inputform=None, fragment=False,
           as_format="default", mapping=None, accept_format=None):
    """Exposes a method to the web.

    By putting the expose decorator on a method, you tell TurboGears that
    the method should be accessible via URL traversal. Additionally, expose
    handles the output processing (turning a dictionary into finished
    output) and is also responsible for ensuring that the request is
    wrapped in a database transaction.

    You can apply multiple expose decorators to a method, if
    you'd like to support multiple output formats. The decorator that's
    listed first in your code without as_format or accept_format is
    the default that is chosen when no format is specifically asked for.
    Any other expose calls that are missing as_format and accept_format
    will have as_format implicitly set to the whatever comes before
    the ":" in the template name (or the whole template name if there
    is no ":". For example, <code>expose("json")</code>, if it's not
    the default expose, will have as_format set to "json".
    
    When as_format is set, passing the same value in the tg_format
    parameter in a request will choose the options for that expose
    decorator. Similarly, accept_format will watch for matching
    Accept headers. You can also use both. expose("json", as_format="json",
    accept_format="text/javascript") will choose JSON output for either
    case: tg_format=json as a parameter or Accept: text/javascript as a
    request header.

    Passing allow_json=True to an expose decorator
    is equivalent to adding the decorator just mentioned.

    Each expose decorator has its own set of options, and each one
    can choose a different template or even template engine (you can
    use Kid for HTML output and Cheetah for plain text, for example).
    See the other expose parameters below to learn about the options
    you can pass to the template engine.

    Take a look at the
    <a href="tests/test_expose-source.html">test_expose.py</a> suite
    for more examples.

    @param template "templateengine:dotted.reference" reference along the
            Python path for the template and the template engine. For
            example, "kid:foo.bar" will have Kid render the bar template in
            the foo package.
    @keyparam format format for the template engine to output (if the
            template engine can render different formats. Kid, for example,
            can render "html", "xml" or "xhtml")
    @keyparam content_type sets the content-type http header
    @keyparam allow_json allow the function to be exposed as json
    @keyparam fragment for template engines (like Kid) that generate
            DOCTYPE declarations and the like, this is a signal to
            just generate the immediate template fragment. Use this
            if you're building up a page from multiple templates or
            going to put something onto a page with .innerHTML.
    @keyparam mapping mapping with options that are sent to the template
            engine
    @keyparam as_format designates which value of tg_format will choose
            this expose.
    @keyparam accept_format which value of an Accept: header will 
            choose this expose.
    @keyparam html deprecated in favor of template
    @keyparam validators deprecated. Maps argument names to validator
            applied to that arg
    @keyparam inputform deprecated. A form object that generates the 
            input to this method
    """
    if html:
        template = html
    if not template:
        template = format
    if format == "json" or (format == None and template == None):
        template = "json"
        allow_json = True
    if content_type is None:
        content_type = config.get("tg.content_type", None)

    if config.get("tg.session.automatic_lock",None) == True:
        cherrypy.session.acquire_lock()

    def entangle(func):
        log.debug("Exposing %s", func)
        log.debug("template: %s, format: %s, allow_json: %s, "
            "content-type: %s", template, format, allow_json, content_type)
        if not getattr(func, "exposed", False):
            def expose(func, *args, **kw):
                accept = cherrypy.request.headers.get('Accept', "").lower()
                if not hasattr(func, "_expose"):
                    _build_rules(func)
                if hasattr(cherrypy.request, "in_transaction"):
                    output = func._expose(func, accept, func._allow_json,
                                *args, **kw)
                else:
                    cherrypy.request.in_transaction = True
                    output = database.run_with_transaction(
                            func._expose, func, accept, func._allow_json,
                            *args, **kw)
                return output
            func.exposed = True
            func._ruleinfo = []
            allow_json_from_config = config.get(
                                        "tg.allow_json", False)
            func._allow_json = allow_json_from_config
        else:
            expose = lambda func, *args, **kw: func(*args, **kw)

        func._ruleinfo.insert(0, dict(as_format = as_format,
            accept_format = accept_format, template = template,
            rulefunc = lambda _func, accept, allow_json,
                    *args, **kw:
                    _execute_func(_func, template, format, content_type,
                                mapping, fragment, args, kw)))

        if allow_json:
            func._allow_json = True

        if inputform or validators:
            import warnings
            warnings.warn(
                "Use a separate decorator validate() rather than passing "
                "arguments validators and/or inputform to decorator "
                "expose().",
                DeprecationWarning, 2)
            func = validate(form=inputform, validators=validators)(func)

        return expose
    return weak_signature_decorator(entangle)

def _execute_func(func, template, format, content_type, mapping, fragment, args, kw):
    """Call controller method and process it's output."""
    if config.get("tg.strict_parameters", False):
	    tg_util.remove_keys(kw, ["tg_random", "tg_format"])
    else:
        args, kw = tg_util.adapt_call(func, args, kw)
    if config.get('server.environment', 'development') == 'development':
        # Only output this in development mode: If it's a field storage object,
        # this means big memory usage, and we don't want that in production
        log.debug("Calling %s with *(%s), **(%s)", func, args, kw)
    output = errorhandling.try_call(func, *args, **kw)
    if isinstance(output, list):
        return output
    assert isinstance(output, basestring) or isinstance(output, dict) \
        or isinstance(output, types.GeneratorType), \
           "Method %s.%s() returned unexpected output. Output should " \
           "be of type basestring, dict or generator." % (
            args[0].__class__.__name__, func.__name__)
    if isinstance(output, dict):
        template = output.pop("tg_template", template)
        format= output.pop("tg_format", format)
    if template and template.startswith("."):
        template = func.__module__[:func.__module__.rfind('.')]+template
    return _process_output(output, template, format, content_type, mapping, fragment)

def flash(message):
    """Set a message to be displayed in the browser on next page display."""
    cherrypy.response.simple_cookie['tg_flash'] = tg_util.to_utf8(message)
    cherrypy.response.simple_cookie['tg_flash']['path'] = '/'

def _get_flash():
    """Retrieve the flash message (if one is set), clearing the message."""
    request_cookie = cherrypy.request.simple_cookie
    response_cookie = cherrypy.response.simple_cookie

    def clearcookie():
        response_cookie["tg_flash"] = ""
        response_cookie["tg_flash"]['expires'] = 0
        response_cookie['tg_flash']['path'] = '/'

    if response_cookie.has_key("tg_flash"):
        message = response_cookie["tg_flash"].value
        response_cookie.pop("tg_flash")
        if request_cookie.has_key("tg_flash"):
            # New flash overrided old one sitting in cookie. Clear that old cookie.
            clearcookie()
    elif request_cookie.has_key("tg_flash"):
        message = request_cookie["tg_flash"].value
        if not response_cookie.has_key("tg_flash"):
            clearcookie()
    else:
        message = None
    if message:
        message = unicode(message, 'utf-8')
    return message

class Controller(object):
    """Base class for a web application's controller.

    Currently, this provides positional parameters functionality
    via a standard default method.
    """

class RootController(Controller):
    """Base class for the root of a web application.

    Your web application should have one of these. The root of
    your application is used to compute URLs used by your app.
    """
    is_app_root = True

    msglog = logging.getLogger('cherrypy.msg')
    msglogfunc = {0: msglog.info, 1: msglog.warning, 2: msglog.error}
    def _cp_log_message(self, msg, context = 'nocontext', severity = 0):
        log = self.msglogfunc[severity]
        text = ''.join((context, ': ', msg))
        log(text)

    accesslog = logging.getLogger("turbogears.access")
    def _cp_log_access(self):
        tmpl = '%(h)s %(l)s %(u)s "%(r)s" %(s)s %(b)s "%(f)s" "%(a)s"'
        try:
            username = cherrypy.request.user_name
            if not username:
                username = "-"
        except AttributeError:
            username = "-"
        s = tmpl % {'h': cherrypy.request.remote_host,
                   'l': '-',
                   'u': username,
                   'r': cherrypy.request.requestLine,
                   's': cherrypy.response.status.split(" ", 1)[0],
                   'b': cherrypy.response.headers.get('Content-Length',
                            '') or "-",
                   'f': cherrypy.request.headers.get('referer', ''),
                   'a': cherrypy.request.headers.get('user-agent', ''),
        }
        self.accesslog.info(s)

Root = RootController

def url(tgpath, tgparams=None, **kw):
    """Computes URLs.

    tgpath can be a list or a string. If the path is absolute (starts
    with a "/"), the server.webpath and the approot of the application
    are prepended to the path. In order for the approot to be
    detected properly, the root object should extend
    controllers.RootController.

    Query parameters for the URL can be passed in as a dictionary in
    the second argument *or* as keyword parameters.
    """
    if not isinstance(tgpath, basestring):
        tgpath = "/".join(list(tgpath))
    if tgpath.startswith("/"):
        if tg_util.request_available():
            check_app_root()
            tgpath = cherrypy.request.app_root + tgpath
        result = config.get("server.webpath", "") + tgpath
    else:
        result = tgpath
    if tgparams is not None:
        tgparams.update(kw)
    else:
        tgparams = kw
    args = []
    for key, value in tgparams.iteritems():
        if value is None:
            continue
        if isinstance(value, unicode):
            value = value.encode("utf8")
        args.append("%s=%s" % (key, urllib.quote(str(value))))
    if args:
        result += "?" + "&".join(args)
    return result

def check_app_root():
    """Sets cherrypy.request.app_root if needed."""
    if hasattr(cherrypy.request, "app_root"):
        return
    found_root = False
    trail = cherrypy.request.object_trail
    top = len(trail) - 1
    # compute the app_root by stepping back through the object
    # trail and collecting up the path elements after the first
    # root we find
    # we can eliminate this if we find a way to use
    # CherryPy's mounting mechanism whenever a new root
    # is hit.
    rootlist = []
    for i in xrange(len(trail) - 1, -1, -1):
        path, obj = trail[i]
        if not found_root and isinstance(obj, RootController):
            if i == top:
                break
            found_root = True
        if found_root and i > 0:
            rootlist.insert(0, path)
    app_root = "/".join(rootlist)
    if not app_root.startswith("/"):
        app_root = "/" + app_root
    if app_root.endswith("/"):
        app_root = app_root[:-1]
    cherrypy.request.app_root = app_root

def redirect(redirect_path, redirect_params=None, **kw):
    """
    Redirect (via cherrypy.HTTPRedirect).
    Raises the exception instead of returning it, this to allow
    users to both call it as a function or to raise it as an exeption.
    """
    raise cherrypy.HTTPRedirect(
                    url(tgpath=redirect_path, tgparams=redirect_params, **kw))

__all__ = ["expose", "validate", "redirect", "flash",
           "Root", "RootController", "Controller",
           "error_handler", "exception_handler",
          ]



from ez_setup import use_setuptools
use_setuptools()
from setuptools import setup, find_packages
from pkg_resources import DistributionNotFound

import sys
import os

if sys.version_info < (2, 3):
    raise SystemExit("Python 2.3 or later is required")

execfile(os.path.join("turbogears", "release.py"))

# setup params
install_requires = [
        "TurboJson >= 0.9.9",
        "TurboCheetah >= 0.9.5",
        "TurboKid >= 1.0.1",
        "CherryPy >= 2.2.1,<3.0.0alpha",
        "simplejson >= 1.3",
        "elementtree >= 1.2.6",
        "PasteScript >= 0.9.7",
        "FormEncode >= 0.7.1",
        "setuptools >= 0.6c2",
        "RuleDispatch >= 0.5a0.dev-r2303",
        "DecoratorTools >= 1.4",
        "ConfigObj >= 4.3.2"]
tgtesttools =  ["nose >= 0.9.1", "SQLAlchemy>=0.3"]
standard = ["SQLObject>=0.8"]

# python 2.5 compatible list
if sys.version_info < (2, 5):
    install_requires.extend([
        "cElementTree >= 1.0.5",
        ])
    tgtesttools.extend([
        "pysqlite"
        ])
    standard = ["SQLObject==bugfix,>=0.7.1dev-r1860,<=0.7.99"]


setup(
    name="TurboGears",
    version=version,
    author=author,
    author_email=email,
    download_url="http://www.turbogears.org/download/filelist.html",
    dependency_links=[
        "http://files.turbogears.org/eggs/",
        ],
    license=license,
    description="front-to-back rapid web development",
    long_description="""
TurboGears brings together four major pieces to create an
easy to install, easy to use web megaframework. It covers
everything from front end (MochiKit JavaScript for the browser,
Kid for templates in Python) to the controllers (CherryPy) to
the back end (SQLObject).

The TurboGears project is focused on providing documentation
and integration with these tools without losing touch
with the communities that already exist around those tools.

TurboGears is easy to use for a wide range of web applications.

The latest development version is available in
<a href="http://www.turbogears.org/svn/turbogears/trunk#egg=turbogears-dev">
the TurboGears subversion repository</a>.""",
    url="http://www.turbogears.org",
    zip_safe=False,
    install_requires = install_requires,
    packages=find_packages(),
    include_package_data=True,
    exclude_package_data={"thirdparty": ["*"]},
    entry_points = """
    [console_scripts]
    tg-admin = turbogears.command:main

    [distutils.commands]
    docs = turbogears.docgen:GenSite

    [paste.paster_create_template]
    tgbase = turbogears.command.quickstart:BaseTemplate
    turbogears = turbogears.command.quickstart:TurbogearsTemplate
    tgbig = turbogears.command.quickstart:TGBig
    tgwidget = turbogears.command.quickstart:TGWidgetTemplate

    [turbogears.command]
    quickstart = turbogears.command.quickstart:quickstart
    sql = turbogears.command.base:SQL
    shell = turbogears.command.base:Shell
    toolbox = turbogears.command.base:ToolboxCommand
    update = turbogears.command.quickstart:update
    i18n = turbogears.command.i18n:InternationalizationTool
    info = turbogears.command.info:InfoCommand

    [turbogears.identity.provider]
    sqlobject = turbogears.identity.soprovider:SqlObjectIdentityProvider
    sqlalchemy= turbogears.identity.saprovider:SqlAlchemyIdentityProvider

    [turbogears.extensions]
    identity = turbogears.identity.visitor
    visit = turbogears.visit

    [turbogears.visit.manager]
    sqlobject = turbogears.visit.sovisit:SqlObjectVisitManager
    sqlalchemy = turbogears.visit.savisit:SqlAlchemyVisitManager

    [turbogears.toolboxcommand]
    widgets = turbogears.toolbox.base:WidgetBrowser
    shell = turbogears.toolbox.shell:WebConsole
    admi18n = turbogears.toolbox.admi18n:Internationalization
    designer = turbogears.toolbox.designer:Designer
    info = turbogears.toolbox.base:Info
    catwalk = turbogears.toolbox.catwalk:CatWalk

    """,
    extras_require = {
        "exp" : ["TGFastData"],
        "standard" : standard,
        "future" : ["Genshi>=0.3", "SQLAlchemy>=0.3"],
        "testtools" : ["nose >= 0.9.1"],
        "tgtesttools" : tgtesttools,
    },
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules'],
    test_suite = 'nose.collector',
    )




# Formatting configuration for locale ca_ES

languages={'gu': 'gujarati', 'gd': u'escoc\xe8s', 'ga': u'irland\xe8s', 'gn': u'guaran\xed', 'gl': 'gallec', 'la': u'llat\xed', 'ln': 'lingala', 'lo': u'laosi\xe0', 'tt': u't\xe0tar', 'tr': 'turc', 'ts': 'tsonga', 'lv': u'let\xf3', 'to': 'tonga', 'lt': u'litu\xe0', 'tk': 'turcman', 'th': 'thai', 'ti': 'tigrinya', 'tg': 'tadjik', 'te': 'telugu', 'ta': u't\xe0mil', 'yi': 'jiddish', 'yo': 'ioruba', 'de': 'alemany', 'da': u'dan\xe8s', 'dz': u'bhutan\xe8s', 'st': 'sotho', 'qu': u'qu\xe8txua', 'el': 'grec', 'eo': 'esperanto', 'en': u'angl\xe8s', 'zh': u'xin\xe9s', 'za': 'zhuang', 'uk': u'ucra\xefn\xe8s', 'eu': 'basc', 'et': u'estoni\xe0', 'es': 'espanyol', 'ru': 'rus', 'rw': 'kinyarwanda', 'rm': u'retorom\xe0nic', 'rn': 'kirundi', 'ro': u'roman\xe8s', 'bn': u'bengal\xed', 'be': u'bielor\xfas', 'bg': u'b\xfalgar', 'ba': 'baixkir', 'wo': u'w\xf2lof', 'jv': u'javan\xe8s', 'bo': u'tibet\xe0', 'bh': 'bihari', 'bi': 'bislama', 'br': u'bret\xf3', 'ja': u'japon\xe8s', 'om': 'oromo (afan)', 'oc': u'occit\xe0', 'tw': 'twi', 'or': 'oriya', 'xh': 'xosa', 'co': 'cors', 'ca': u'catal\xe0', 'cy': u'gal\xb7l\xe8s', 'cs': 'txec', 'ps': 'paixto', 'pt': u'portugu\xe8s', 'tl': u'tag\xe0log', 'pa': 'panjabi', 'vi': 'vietnamita', 'pl': u'polon\xe8s', 'hy': 'armeni', 'hr': 'croat', 'iu': 'inuktitut', 'hu': u'hongar\xe8s', 'hi': 'hindi', 'ha': 'hausa', 'he': 'hebreu', 'mg': 'malgaix', 'uz': 'uzbek', 'ml': 'malaialam', 'mo': 'moldau', 'mn': 'mongol', 'mi': 'maori', 'ik': 'inupiak', 'mk': 'macedoni', 'ur': u'urd\xfa', 'mt': u'malt\xe8s', 'ms': 'malai', 'mr': 'marathi', 'ug': 'uigur', 'my': u'birm\xe0', 'aa': u'\xe0far', 'ab': 'abkhaz', 'ss': 'siswati', 'af': 'afrikaans', 'tn': 'tswana', 'sw': 'swahili', 'is': u'island\xe8s', 'am': u'amh\xe0ric', 'it': u'itali\xe0', 'sv': 'suec', 'ia': 'interlingua', 'as': u'assam\xe8s', 'ar': u'\xe2rab', 'su': u'sundan\xe8s', 'zu': 'zulu', 'ay': 'aimara', 'az': u'\xe0zeri', 'ie': 'interlingue', 'id': 'indonesi', 'sk': 'eslovac', 'nl': u'neerland\xe8s', 'no': 'noruec', 'na': u'nauru\xe0', 'ne': u'nepal\xe8s', 'vo': 'volapuk', 'fr': u'franc\xe8s', 'sm': u'samo\xe0', 'fy': u'fris\xf3', 'fa': 'persa', 'fi': u'fin\xe8s', 'fj': u'fiji\xe0', 'sa': u's\xe0nscrit', 'fo': u'fero\xe8s', 'ka': u'georgi\xe0', 'kk': 'kazakh', 'sr': 'serbi', 'sq': u'alban\xe8s', 'ko': u'core\xe0', 'kn': 'kannada', 'km': u'cambodj\xe0', 'kl': u'greenland\xe8s', 'ks': 'caixmiri', 'si': u'sinhal\xe8s', 'sh': 'serbo-croat', 'so': 'somali', 'sn': 'shona', 'ku': 'kurd', 'sl': u'eslov\xe8', 'ky': u'kirgu\xeds', 'sg': 'sango', 'sd': 'sindhi'}

countries={'BD': 'Bangla Desh', 'BE': u'B\xe8lgica', 'BF': 'Burkina Faso', 'BG': u'Bulg\xe0ria', 'BA': u'B\xf2snia i Hercegovina', 'BB': 'Barbados', 'BM': 'Bermudes', 'BN': 'Brunei', 'BO': u'Bol\xedvia', 'BH': 'Bahrain', 'BI': 'Burundi', 'BJ': 'Benin', 'BT': 'Bhutan', 'JM': 'Jamaica', 'BW': 'Botswana', 'BR': 'Brasil', 'BS': 'Bahames', 'BY': u'Bielor\xfassia', 'BZ': 'Belize', 'RU': u'R\xfassia', 'RW': 'Rwanda', 'TL': 'Timor', 'TM': 'Turkmenistan', 'TJ': 'Tadjikistan', 'RO': 'Romania', 'TK': 'Tokelau', 'GW': 'Guinea Bissau', 'GT': 'Guatemala', 'GR': u'Gr\xe8cia', 'GQ': 'Guinea Equatorial', 'GP': 'Guadeloupe', 'JP': u'Jap\xf3', 'GY': 'Guyana', 'GF': 'Guaiana Francesa', 'GE': u'Ge\xf2rgia', 'GB': 'Regne Unit', 'GA': 'Gabon', 'GN': 'Guinea', 'GM': u'G\xe0mbia', 'GH': 'Ghana', 'OM': 'Oman', 'TN': u'Tun\xedsia', 'JO': u'Jord\xe0nia', 'SP': u'S\xe8rbia', 'HR': u'Cro\xe0cia', 'HT': u'Hait\xed', 'HU': 'Hongria', 'HK': 'Hong Kong', 'HN': 'Hondures', 'VE': u'Vene\xe7uela', 'PR': 'Puerto Rico', 'PT': 'Portugal', 'PY': 'Paraguai', 'IQ': 'Iraq', 'PA': u'Panam\xe0', 'PF': u'Polin\xe8sia Francesa', 'PG': 'Papua Nova Guinea', 'PE': u'Per\xfa', 'PK': 'Pakistan', 'PH': 'Filipines', 'PL': u'Pol\xf2nia', 'ZM': u'Z\xe0mbia', 'EH': u'S\xe0hara Occidental', 'EE': u'Est\xf2nia', 'EG': 'Egipte', 'ZA': u'Sud-\xe0frica', 'EC': 'Equador', 'IT': u'It\xe0lia', 'VN': 'Vietnam', 'ET': u'Eti\xf2pia', 'SO': u'Som\xe0lia', 'ZW': 'Zimbabwe', 'ES': 'Espanya', 'ER': 'Eritrea', 'MD': u'Mold\xe0via', 'MG': 'Madagascar', 'MA': 'Marroc', 'MC': u'M\xf2naco', 'UZ': 'Uzbekistan', 'MM': 'Myanmar', 'ML': 'Mali', 'MN': u'Mong\xf2lia', 'MK': u'Maced\xf2nia', 'MU': 'Maurici', 'MT': 'Malta', 'MQ': 'Martinica', 'MS': 'Montserrat', 'MR': u'Maurit\xe0nia', 'UG': 'Uganda', 'MY': u'Mal\xe0isia', 'MX': u'M\xe8xic', 'IL': 'Israel', 'FR': u'Fran\xe7a', 'FI': u'Finl\xe0ndia', 'FJ': 'Fiji', 'FM': u'Micron\xe8sia', 'NI': 'Nicaragua', 'NL': u'Pa\xefsos Baixos', 'NO': 'Noruega', 'NA': u'Nam\xedbia', 'VU': 'Vanuatu', 'NC': u'Nova Caled\xf2nia', 'NE': u'N\xedger', 'NG': u'Nig\xe8ria', 'NZ': 'Nova Zelanda', 'NP': 'Nepal', 'NU': 'Niue', 'CI': "Costa d'Ivori", 'CH': 'Switzerland', 'CO': u'Col\xf2mbia', 'CN': 'Xina', 'CM': 'Camerun', 'CL': 'Xile', 'CA': u'Canad\xe0', 'CG': 'Congo', 'CF': u'Rep\xfablica Centrafricana', 'CZ': u'Rep\xfablica Txeca', 'CY': 'Xipre', 'CR': 'Costa Rica', 'CV': 'Cap Verd', 'CU': 'Cuba', 'SZ': u'Swazil\xe0ndia', 'SY': u'S\xedria', 'KG': 'Kirgizistan', 'KE': 'Kenya', 'SR': 'Surinam', 'KI': 'Kiribati', 'KH': 'Cambodja', 'SV': 'El Salvador', 'KM': 'Comores', 'SK': u'Eslov\xe0quia', 'KR': 'Corea del Sud', 'SI': u'Eslov\xe8nia', 'KP': 'Corea del Nord', 'KW': 'Kuwait', 'SN': 'Senegal', 'SL': 'Sierra Leone', 'SC': 'Seychelles', 'KZ': 'Kazakhstan', 'SA': u'Ar\xe0bia Saud\xed', 'SG': 'Singapur', 'SE': u'Su\xe8cia', 'SD': 'Sudan', 'DO': u'Rep\xfablica Dominicana', 'DM': 'Dominica', 'DJ': 'Djibouti', 'DK': 'Dinamarca', 'VG': u'Illes Verges Brit\xe0niques', 'DE': 'Alemanya', 'YE': 'Iemen', 'DZ': u'Alg\xe8ria', 'US': 'Estats Units', 'UY': 'Uruguai', 'YU': u'Iugosl\xe0via', 'YT': 'Mayotte', 'LB': u'L\xedban', 'LA': 'Laos', 'TW': 'Taiwan', 'TT': 'Trinitat i Tobago', 'TR': 'Turquia', 'LK': 'Sri Lanka', 'LI': 'Liechtenstein', 'LV': u'Let\xf2nia', 'TO': 'Tonga', 'LT': u'Litu\xe0nia', 'LU': 'Luxemburg', 'LR': u'Lib\xe8ria', 'LS': 'Lesotho', 'TH': u'Tail\xe0ndia', 'TF': 'Territoris Meridionals Francesos', 'TG': 'Togo', 'TD': 'Txad', 'LY': u'L\xedbia', 'VA': u'Vatic\xe0', 'AE': u'Uni\xf3 dels Emirats \xc0rabs', 'AD': 'Andorra', 'AF': 'Afganistan', 'AI': 'Anguilla', 'VI': 'Illes Verges dels USA', 'IS': u'Isl\xe0ndia', 'IR': 'Iran', 'AM': u'Arm\xe8nia', 'AL': u'Alb\xe0nia', 'AO': 'Angola', 'AN': 'Antilles Holandeses', 'AR': 'Argentina', 'AU': u'Austr\xe0lia', 'AT': u'\xc2ustria', 'AW': 'Aruba', 'IN': u'\xcdndia', 'TZ': u'Tanz\xe0nia', 'AZ': 'Azerbaidjan', 'IE': 'Irlanda', 'ID': u'Indon\xe8sia', 'UA': u'Ucra\xefna', 'QA': 'Qatar', 'MZ': u'Mo\xe7ambic'}

months=['gener', 'febrer', u'mar\xe7', 'abril', 'maig', 'juny', 'juliol', 'agost', 'setembre', 'octubre', 'novembre', 'desembre']

abbrMonths=['gen.', 'feb.', u'mar\xe7', 'abr.', 'maig', 'juny', 'jul.', 'ag.', 'set.', 'oct.', 'nov.', 'des.']

days=['dilluns', 'dimarts', 'dimecres', 'dijous', 'divendres', 'dissabte', 'diumenge']

abbrDays=['dl.', 'dt.', 'dc.', 'dj.', 'dv.', 'ds.', 'dg.']

dateFormats={'medium': '%d/%m/%Y', 'full': '%%(dayname)s %d %%(monthname)s %Y', 'long': '%d %%(monthname)s %Y', 'short': '%d/%m/%y'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}


"""
General i18n utility functions.
"""

import os
import urllib
import httplib
import re
import cherrypy
from turbogears import config
from turbogears.util import parse_http_accept_header, request_available

def google_translate(from_lang, to_lang, text):

    params = urllib.urlencode({"langpair":"%s|%s" %(from_lang, to_lang), "text":text,
    "ie":"UTF8", "oe":"UTF8"})
    conn = httplib.HTTPConnection("translate.google.com")
    conn.request("POST", "/translate_t", params)

    resp = conn.getresponse()
    s = resp.read()
    conn.close()

    match = re.compile('<textarea name=q.*?>(.*?)</textarea>',
                       re.DOTALL).search(s)
    data = match.groups()[0]
    return unicode(data, "utf-8").strip()

def lang_in_gettext_format(lang):
    if len(lang) > 2:
        country = lang[3:].upper()
        lang = lang[:2] + "_" + country
    return lang

def get_accept_languages(accept):
    """Returns a list of languages, by order of preference, based on an
    HTTP Accept-Language string.See W3C RFC 2616
    (http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html) for specification.
    """
    langs = parse_http_accept_header(accept)
    for index, lang in enumerate(langs):
        langs[index] = lang_in_gettext_format(lang)
    return langs

def get_locale(locale=None):
    """
    Returns user locale, using _get_locale or app-specific locale lookup function.
    """
    if not locale:
        get_locale_f = config.get("i18n.get_locale", _get_locale)
        locale = get_locale_f()
    return locale

def _get_locale():
    """Default function for returning locale. First looks in session for locale key,
    then checks the HTTP Accept-Language header, and finally checks the config default
    locale setting. This can be replaced by your own function by setting cherrypy
    config setting i18n.get_locale to your function name.
    """
    if not request_available():
        return config.get("i18n.default_locale", "en")
    
    if config.get("session_filter.on", False):
        locale_key = config.get("i18n.session_key", "locale")
        locale = cherrypy.session.get(locale_key)
        if locale:
            return locale
    browser_accept_lang = _get_locale_from_accept_header()
    return browser_accept_lang or config.get("i18n.default_locale", "en")

def _get_locale_from_accept_header():
    """
    Checks HTTP Accept-Language header to find preferred language if any.
    """
    try:
        header = cherrypy.request.headers.get("Accept-Language")
        if header:
            accept_languages = get_accept_languages(header)
            if accept_languages:
                return accept_languages[0]
    except AttributeError:
        pass

def set_session_locale(locale):
    """
    Sets the i18n session locale.

    Raises an error if session support is not enabled.
    """
    cherrypy.session[config.get("i18n.session_key", "locale")] = locale






import turbogears
import pkg_resources
import sys
from cStringIO import StringIO
import logging
import re

testfile = pkg_resources.resource_filename(__name__, "configfile.cfg")

rfn = pkg_resources.resource_filename

logout = StringIO()
logging.logout = logout

def test_update_from_package():
    turbogears.update_config(modulename="turbogears.tests.config")
    assert turbogears.config.get("foo.bar") == "BAZ!"
    print turbogears.config.get("my.static")
    assert turbogears.config.get("my.static").endswith(
                                            "turbogears/tests/static")
    assert turbogears.config.get("static_filter.on", path="/static") == True

def test_update_from_both():
    turbogears.update_config(configfile = testfile, 
        modulename="turbogears.tests.config")
    print turbogears.config.get("foo.bar")
    assert turbogears.config.get("foo.bar") == "blurb"
    assert turbogears.config.get("tg.something") == 10
    print turbogears.config.get("test.dir")
    assert turbogears.config.get("test.dir").endswith("turbogears/tests")

callnum = 0

def windows_filename(*args, **kw):
    global callnum
    callnum += 1
    if callnum > 1:
        return "c:\\foo\\bar\\"
    else:
        return rfn(*args, **kw)

def test_windows_filenames():
    pkg_resources.resource_filename = windows_filename
    turbogears.update_config(configfile = testfile, 
        modulename="turbogears.tests.config")
    testdir = turbogears.config.get("test.dir")
    print testdir
    assert testdir == "c:/foo/bar"

def test_logging_config():
    logout.truncate(0)
    log = logging.getLogger("turbogears.tests.test_config.logconfig")
    log.info("Testing")
    logged = logout.getvalue()
    print "Logged: %s" % logged
    assert re.match(r'F1 \d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d,\d\d\d INFO '
                    'Testing', logged)
    assert turbogears.config.get("tg.new_style_logging", False)
        
def teardown_module():
    pkg_resources.resource_filename = rfn



# Formatting configuration for locale pt_PT

languages={'lv': u'let\xe3o', 'ar': u'\xe1rabe', 'sl': 'esloveno', 'cs': 'checo', 'et': u'est\xf3nio', 'pl': 'polaco'}

countries={'BA': u'B\xf3snia-Herzegovina', 'YE': u'I\xe9men', 'BJ': 'Benim', 'HK': u'Hong Kong - Regi\xe3o Administrativa Especial da China', 'UM': 'Ilhas Minor Outlying (E.U.A)', 'GS': 'Ilhas South Georgia e South Sandwich', 'FK': 'Ilhas Falkland', 'YU': u'Jugosl\xe1via', 'BY': u'Bielorr\xfassia', 'PS': u'Territ\xf3rio Palestiniano', 'AZ': u'Azerbeij\xe3o', 'CM': u'Camar\xf5es', 'LA': u'Lao, Rep\xfablica Popular Democr\xe1tica', 'TW': u'Formosa, Prov\xedncia Chinesa', 'NC': u'Nova Caled\xf3nia', 'LV': u'Let\xf3nia', 'RE': 'Reunion', 'TM': u'Turquemenist\xe3o', 'TJ': u'Tajiquist\xe3o', 'PG': u'Papua Nova Guin\xe9', 'TF': u'Territ\xf3rios Franceses a Sul', 'TD': 'Tchade', 'RO': u'Rom\xe9nia', 'TC': 'Ilhas Turcas e Caicos', 'PL': u'Pol\xf3nia', 'GW': u'Guin\xe9-Bissau', 'VA': u'Santa S\xe9 (Estado da Cidade do Vaticano)', 'VC': 'Saint Vincent e Grenadines', 'AE': u'Emiratos \xc1rabes Unidos', 'EH': 'Sahara Ocidental', 'AG': u'Ant\xedgua e Barbuda', 'VG': u'Ilhas Virgin Brit\xe2nicas', 'EE': u'Est\xf3nia', 'VI': 'Ilhas Virgin E.U.A.', 'EG': 'Egipto', 'AM': u'Arm\xe9nia', 'VN': 'Vietname', 'AQ': u'Ant\xe1rctica', 'CZ': u'Rep\xfablica Checa', 'CX': 'Ilha do Natal', 'GL': u'Gronel\xe2ndia', 'ER': 'Eritreia', 'MD': u'Mold\xe1via, Rep\xfablica da', 'MG': u'Madag\xe1scar', 'KG': u'Quirguizist\xe3o', 'MC': u'M\xf3naco', 'KE': u'Qu\xe9nia', 'UZ': u'Uzbaquist\xe3o', 'MO': u'Macau - Regi\xe3o Administrativa Especial da China', 'KH': 'Camboja', 'KN': 'Saint Kitts e Nevis', 'MK': u'Maced\xf3nia, Rep\xfablica da', 'MU': u'Maur\xedcias', 'KR': 'Coreia do Sul', 'SI': u'Eslov\xe9nia', 'KP': 'Coreia do Norte', 'MP': 'Ilhas Mariana do Norte', 'SM': u'S\xe3o Marino', 'SC': 'Seicheles', 'KZ': u'Cazaquist\xe3o', 'KY': u'Ilhas Caim\xe3o', 'SG': 'Singapura'}

months=['janeiro', 'fevereiro', u'mar\xe7o', 'abril', 'maio', 'junho', 'julho', 'agosto', 'setembro', 'outubro', 'novembro', 'dezembro']

abbrMonths=['jan', 'fev', 'mar', 'abr', 'mai', 'jun', 'jul', 'ago', 'set', 'out', 'nov', 'dez']

days=['segunda-feira', u'ter\xe7a-feira', 'quarta-feira', 'quinta-feira', 'sexta-feira', u's\xe1bado', 'domingo']

abbrDays=['seg', 'ter', 'qua', 'qui', 'sex', u's\xe1b', 'dom']

dateFormats={'medium': '%Y/%m/%d', 'full': '%%(dayname)s, %d de %%(monthname)s de %Y', 'long': '%d de %%(monthname)s de %Y', 'short': '%y/%m/%d'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}


import turbogears
import cherrypy

from turbogears import widgets, testutil

def test_table_widget_js():
    """
    The TableForm Widget can require JavaScript and CSS resources. Addresses
    ticket #425. Should be applicable to any widget.
    """
    class MyTableWithJS(widgets.TableForm):
        javascript = [widgets.JSLink(mod=widgets.static, name="foo.js"),
                      widgets.JSSource("alert('hello');")]
        css = [widgets.CSSLink(mod=widgets.static, name="foo.css")]

    form = MyTableWithJS(fields=[widgets.TextField(name='title')]) 

    class MyRoot(turbogears.controllers.RootController):
        def test(self):
            return dict(form=form)
        test = turbogears.expose(template=".form")(test)

    cherrypy.root = MyRoot()
    testutil.createRequest("/test")
    output = cherrypy.response.body[0]
    print output
    assert 'foo.js' in output
    assert "alert('hello');" in output
    assert 'foo.css' in output

def test_calendardatepicker_js():

    class MyRoot(turbogears.controllers.RootController):
        def test(self):
            return dict(widget=widgets.CalendarDatePicker())
        test = turbogears.expose(template=".widget")(test)
            
    cherrypy.root = MyRoot()

    # testing default language (en)
    testutil.createRequest("/test")
    output = cherrypy.response.body[0]
    print output
    assert 'calendar/calendar.js' in output
    assert 'calendar/calendar-setup.js' in output
    assert 'calendar/lang/calendar-en.js' in output

    # testing french language
    testutil.createRequest("/test", headers={"Accept-Language":"fr"})
    output = cherrypy.response.body[0]
    print output
    assert 'calendar/lang/calendar-fr.js' in output



"""
Functions for use with Kid template
"""
from kid.pull import START, TEXT, END

import turbogears
# use plain_gettext because Kid template's strings always evaluated immediately
from turbogears.i18n.tg_gettext import plain_gettext as gettext
from turbogears.i18n.utils import google_translate

def translate(item, attr=None):
    """Translates the text of element plus the text of all child elements. If attr is present 
    this is used to provide the locale name; if not then value provided by get_locale is used.
    For example:
    
    <div lang="">
        this is a test
        <a href="de">testing</a>
    </div>
    
    The string 'this is a test' is rendered by the locale provided by get_locale, the 
    string 'testing' by the German locale.
    
    Possible use in Kid template:
    <?python
    from turbogears.i18n import translate
    ?>
    <translate xmlns:py="http://purl.org/kid/ns#" py:match="item.attrib.has_key('lang')" 
    py:replace="translate(item)"/>
    <h1 lang="">Welcome!</h1>
    @param item: element to be translated
    @type item: ElementTree element
    @param attr: attribute name used to store locale, if any
    """
    if attr is None:attr = turbogears.config.get("i18n.templateLocaleAttribute", "lang")    
    translate_all(item, item.get(attr), attr)
    return item

def __translate_text(text, lang):
    prefix = ''
    postfix = ''
    if len(text) > 0 and text[0].isspace(): prefix = text[0]
    if len(text) > 1 and text[-1].isspace(): postfix = text[-1]
    return prefix + gettext(text.strip(), lang) + postfix

def translate_all(tree, lang, attr, inroot=True):
    """Recursive function to translate all text in child elements
    @param tree: parent ElementTree element
    @param lang: language setting
    @param attr: attribute name used to store locale
    """

    if tree.text:
        tree.text = __translate_text(tree.text, lang)
    if tree.tail and not inroot:
        # Don't translate tail of root. It is beyond the scope of the lang attr 
        tree.tail = __translate_text(tree.tail, lang)
    for element in tree:
        # check overriding lang attribute
        if element.get(attr):lang = element.get(attr)
        translate_all(element, lang, attr, False)

def i18n_filter(stream, template, locale=None):
    """Kid template filter which calls translates all elements matching language 
    attribute(set in configuration as i18n.templateLocaleAttribute, default 'lang')
    """
    
    lang_attr = turbogears.config.get("i18n.templateLocaleAttribute", "lang")    
    locales=[locale]
    
    for ev, item in stream:
                
        if ev==START:        
            l = item.get(lang_attr)
            if l:    
                locale = l
                locales.append(l)                
        elif ev==TEXT:
            prefix = ''
            postfix = ''
            if len(item) > 0 and item[0] == ' ': prefix =' '
            if len(item) > 1 and item[-1] == ' ': postfix =' '
            
            text = item.strip()
            if text:
                item = gettext(text, locale)     
                item = prefix + item + postfix
        elif ev==END:
            if item.get(lang_attr):
                locales.pop()
                locale = locales[-1]
            
        yield (ev, item)



import cherrypy
import random
from datetime import *

import turbogears
from turbogears import identity
from turbogears.util import load_class
from turbogears.database import session
from turbojson.jsonify import *

import logging
log = logging.getLogger("turbogears.identity.saprovider")

try:
    set, frozenset
except NameError:
    from sets import Set as set, ImmutableSet as frozenset

# Global class references --
# these will be set when the Provider is initialised.
user_class = None
group_class = None
permission_class = None
visit_class = None

class SqlAlchemyIdentity(object):

    def __init__(self, visit_key, user=None):
        if user:
            self._user = user
        self.visit_key = visit_key

    def _get_user(self):
        try:
            return self._user
        except AttributeError:
            # User hasn't already been set
            pass
        # Attempt to load the user. After this code executes, there *WILL* be
        # a _user attribute, even if the value is None.
        visit = session.query(visit_class).get_by(visit_key = self.visit_key)
        if not visit:
            self._user = None
            return None
        self._user = session.query(user_class).get(visit.user_id)
        return self._user
    user = property(_get_user)

    def _get_user_name(self):
        if not self.user:
            return None
        return self.user.user_name
    user_name = property(_get_user_name)

    def _get_anonymous(self):
        return not self.user
    anonymous = property(_get_anonymous)

    def _get_permissions(self):
        try:
            return self._permissions
        except AttributeError:
            # Permissions haven't been computed yet
            pass
        if not self.user:
            self._permissions = frozenset()
        else:
            self._permissions = frozenset([
                    p.permission_name for p in self.user.permissions])
        return self._permissions
    permissions = property(_get_permissions)

    def _get_groups(self):
        try:
            return self._groups
        except AttributeError:
            # Groups haven't been computed yet
            pass
        if not self.user:
            self._groups = frozenset()
        else:
            self._groups = frozenset([g.group_name for g in self.user.groups])
        return self._groups
    groups = property(_get_groups)

    def logout(self):
        '''
        Remove the link between this identity and the visit.
        '''
        if not self.visit_key:
            return
        try:
            visit = session.query(visit_class).get_by(visit_key=self.visit_key)
            session.delete(visit)
            # Clear the current identity
            anon = SqlAlchemyIdentity(None,None)
            identity.set_current_identity(anon)
        except:
            pass
        else:
            session.flush()


class SqlAlchemyIdentityProvider(object):
    '''
    IdentityProvider that uses a model from a database (via SQLAlchemy).
    '''

    def __init__(self):
        super(SqlAlchemyIdentityProvider, self).__init__()
        get=turbogears.config.get

        global user_class, group_class, permission_class, visit_class

        user_class_path = get("identity.saprovider.model.user",
                              None)
        user_class = load_class(user_class_path)
        group_class_path = get("identity.saprovider.model.group",
                                None)
        group_class = load_class(group_class_path)
        permission_class_path = get("identity.saprovider.model.permission",
                                    None)
        permission_class = load_class(permission_class_path)
        visit_class_path = get("identity.saprovider.model.visit",
                               None)
        log.info("Loading: %s", visit_class_path)
        visit_class = load_class(visit_class_path)
        # Default encryption algorithm is to use plain text passwords
        algorithm = get("identity.saprovider.encryption_algorithm", None)
        self.encrypt_password = lambda pw: \
                                    identity._encrypt_password(algorithm, pw)

    def create_provider_model(self):
        '''
        Create the database tables if they don't already exist.
        '''
        user_class.mapper.local_table.create(checkfirst=True)
        group_class.mapper.local_table.create(checkfirst=True)
        permission_class.mapper.local_table.create(checkfirst=True)
        visit_class.mapper.local_table.create(checkfirst=True)

    def validate_identity(self, user_name, password, visit_key):
        '''
        Look up the identity represented by user_name and determine whether the
        password is correct.

        Must return either None if the credentials weren't valid or an object
        with the following properties:
            user_name: original user name
            user: a provider dependant object (TG_User or similar)
            groups: a set of group IDs
            permissions: a set of permission IDs
        '''
        user = session.query(user_class).get_by(user_name=user_name)
        if not user:
            log.warning("No such user: %s", user_name)
            return None
        if not self.validate_password(user, user_name, password):
            log.info("Passwords don't match for user: %s", user_name)
            return None

        log.info("associating user (%s) with visit (%s)", user.user_name,
                  visit_key)
        # Link the user to the visit
        link = session.query(visit_class).get_by(visit_key=visit_key)
        if not link:
            link = visit_class(visit_key=visit_key, user_id=user.user_id)
            session.save(link)
        else:
            link.user_id = user.user_id
        session.flush()
        return SqlAlchemyIdentity(visit_key, user)

    def validate_password(self, user, user_name, password):
        '''
        Check the supplied user_name and password against existing credentials.
        Note: user_name is not used here, but is required by external
        password validation schemes that might override this method.
        If you use SqlAlchemyIdentityProvider, but want to check the passwords
        against an external source (i.e. PAM, LDAP, Windows domain, etc),
        subclass SqlAlchemyIdentityProvider, and override this method.
        '''
        return user.password == self.encrypt_password(password)

    def load_identity(self, visit_key):
        '''
        Lookup the principal represented by user_name.
        Return None if there is no principal for the given user ID.

        Must return an object with the following properties:
            user_name: original user name
            user: a provider dependant object (TG_User or similar)
            groups: a set of group IDs
            permissions: a set of permission IDs
        '''
        return SqlAlchemyIdentity(visit_key)

    def anonymous_identity(self):
        '''
        Must return an object with the following properties:
            user_name: original user name
            user: a provider dependant object (TG_User or similar)
            groups: a set of group IDs
            permissions: a set of permission IDs
        '''
        return SqlAlchemyIdentity(None)

    def authenticated_identity(self, user):
        '''
        Constructs Identity object for user that has no associated visit_key.
        '''
        return SqlAlchemyIdentity(None, user)



import unittest
from turbogears import testutil
import turbogears
import cherrypy
import time

def cookie_header(morsel):
    """Returns a dict containing cookie information to pass to a server."""
    return {'Cookie': morsel.output(header="")[1:]}

class VisitRoot(turbogears.controllers.RootController):

    def index(self):
        pass
    index = turbogears.expose()(index)

class TestVisit(unittest.TestCase):

    def setUp(self):
        self._visit_on = turbogears.config.get('visit.on', False)
        turbogears.config.update({'visit.on': True})
        self.cookie_name = turbogears.config.get("visit.cookie.name", 'tg-visit')
        cherrypy.root = VisitRoot()

    def tearDown(self):
        turbogears.config.update({'visit.on': self._visit_on})

    def test_visit_response(self):
        "Test if the visit cookie is set in cherrypy.response."
        testutil.create_request("/")
        assert cherrypy.response.simple_cookie.has_key(self.cookie_name)
        # the following command shuts down the visit framework properly
        # the test still passes without it, but exceptions are thrown later
        # once nose wants to quit.
        turbogears.startup.stopTurboGears()

    def test_new_visit(self):
        "Test that we can see a new visit on the server."
        testutil.create_request("/")
        assert turbogears.visit.current().is_new
        turbogears.startup.stopTurboGears()

    def test_old_visit(self):
        "Test if we can track a visitor over time."
        testutil.create_request("/")
        morsel = cherrypy.response.simple_cookie[self.cookie_name] #first visit's cookie
        testutil.create_request("/", headers=cookie_header(morsel))
        assert not turbogears.visit.current().is_new
        turbogears.startup.stopTurboGears()

    def test_cookie_expires(self):
        "Test if the visit timeout mechanism works."
        turbogears.config.update({'visit.timeout':1.0/60.0})  # set expiration to one second
        testutil.create_request("/")
        morsel = cherrypy.response.simple_cookie[self.cookie_name]
        time.sleep(3)  # 3 seconds
        testutil.create_request("/", headers=cookie_header(morsel))
        assert cherrypy.response.simple_cookie[self.cookie_name].value != morsel.value, 'cookie values should not match'
        assert turbogears.visit.current().is_new, 'this should be a new visit, as the cookie has expired'
        turbogears.startup.stopTurboGears()



# Formatting configuration for locale nn

languages={'nb': u'norsk bokm\xe5l', 'nn': 'norsk nynorsk', 'no': 'norsk'}

countries={'NO': 'Noreg'}

months=['januar', 'februar', 'mars', 'april', 'mai', 'juni', 'juli', 'august', 'september', 'oktober', 'november', 'desember']

abbrMonths=['jan', 'feb', 'mar', 'apr', 'mai', 'jun', 'jul', 'aug', 'sep', 'okt', 'nov', 'des']

days=[u'm\xe5ndag', 'tysdag', 'onsdag', 'torsdag', 'fredag', 'laurdag', 'sundag']

abbrDays=[u'm\xe5', 'ty', 'on', 'to', 'fr', 'la', 'su']

dateFormats={'medium': '%d. %%(abbrmonthname)s. %Y', 'full': '%%(dayname)s %d. %%(monthname)s %Y', 'long': '%d. %%(monthname)s %Y', 'short': '%d.%m.%y'}

numericSymbols={'group': u'\xa0', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}


import turbogears
import sqlobject
from sqlobject.sqlbuilder import *
import cherrypy

class Browse(object):
    def __getattr__(self,attrib):
        """Delegate basic methods to CatWalk"""
        return getattr(self.catwalk,attrib)


    def list_view(sql_class_name,page_size=10,offset=10,fields=[],filters=[],sort_by=None):
        pass 

    def index(self,object_name,start=0,page_size=10,context='',filters=''):
        total = 0
        headers = []
        rows = []
        start = int(start)
        page_size = int(page_size)
        if not context: context = object_name
        headers = self.column_headers(object_name,context)
        headers = [header for header in headers if header['visible']]
        total,rows = self.rows_for_model(object_name,start,page_size,filters)

        return dict(object_name=object_name,
                    start=start,
                    page_size=page_size,
                    total=total,
                    headers= headers,
                    rows = rows 
                   )
    index = turbogears.expose(template='turbogears.toolbox.catwalk.browse_grid',
                              allow_json=True)(index)

    def columns(self,object_name,context=''):
        if not context:context = object_name
        return dict(object_name=object_name,
                    context = context,
                    columns=self.extended_column_headers(object_name,context))
    columns = turbogears.expose(template='turbogears.toolbox.catwalk.columns',
                                allow_json=True)(columns)

    def save_columns(self,object_name,context,order,hidden_columns,updated_fk_labels):
        self.save_column_order(context,order)
        self.hide_columns(context,hidden_columns)
        if updated_fk_labels:
            for updated_fk_label in updated_fk_labels.split('|'):
                object,column_name = updated_fk_label.split(':')
                self.column_label_for_object(object,column_name)
        return "<script>parent.cat_browse.columns_saved('%s','%s');</script>"% (object_name,context)
    save_columns= turbogears.expose(allow_json=True)(save_columns)
        
    def extended_column_headers(self,object_name,context):
        cols = [{'name':'id','title':'#','type':'SOInteger'}]
        cols.extend(self.column_labels(object_name,extended=True))
        cols.extend(self.join_labels(object_name,extended=True))
        cols = self.arrange_columns(cols,context)
        return cols

    def column_headers(self,object_name,context):
        cols = [{'name':'id','title':'#'}]
        cols.extend(self.column_labels(object_name))
        cols.extend(self.join_labels(object_name))
        cols = self.arrange_columns(cols,object_name)
        return cols

    def arrange_columns(self,headers,context):
        #arrange order and visibility 
        hidden_columns = self.load_columns_visibility_state(context)
        order = self.load_column_order(context)
        for col in headers:
            col['visible'] = True
            if col['name'] in hidden_columns: col['visible'] = False

        if not order:return headers
        c={}
        for col in headers:
            c[col['name']] = col
            if col['name'] not in order:order.append(col['name'])
        rearrenged = []
        for name in order:
            if name not in c.keys():continue
            rearrenged.append(c[name])
        return rearrenged

    def prepare_filter(self,obj,filter):
        for column in obj.sqlmeta.columns.values():
            if column.origName == filter[0]:
                return getattr(obj.q,column.name) == filter[1]

        #if we got so far we couldn't find the column, bark at the moon
        msg='filter_column_error. Could not find the column for filter:%s'% filter[0] 
        raise cherrypy.HTTPRedirect(turbogears.url('/error', msg=msg))

    def filtered_query(self,obj,filters):
        if not ':' in filters: #there should at least be a semicolon
            msg='filter_format_error. The format is column_name:value, not %s'% filters 
            raise cherrypy.HTTPRedirect(turbogears.url('/error', msg=msg))

        filters = [filter.split(':') for filter in filters.split(',')]
        conditions =  tuple([self.prepare_filter(obj,filter) for filter in filters])
        return obj.select(AND(* conditions ))
            
    def rows_for_model(self,object_name,start,page_size,filters):
        ids = []
        rows = {} 
        obj = self.load_object(object_name)

        if filters:
            query = self.filtered_query(obj,filters) 
        else:
            query = obj.select()

        total = query.count()
        if page_size:
            results = query[start:start+page_size]
        else:
            results = query[start:]

        for result in results:
            rows[result.id] = self.fields(object_name,result)
        
        relations = self.relation_values(object_name,rows)
        rows = self.merge_relation_values(rows,relations)
        rows = self.foreign_key_alias_value(object_name,rows)
        return total,rows

    def relation_values(self,object_name,rows):
        joins = {}
        ids = rows.keys()
        if not ids: return joins

        obj = self.load_object(object_name)
        conn = obj._connection
        for column in obj.sqlmeta.joins:
            query = None
            coltype = self.get_column_type(column)
            if coltype in ('SOMultipleJoin', 'SOSQLMultipleJoin'):
                query = conn.sqlrepr( Select( [
                                         column.soClass.q.id,
                                         func.Count(column.otherClass.q.id)
                                         ],
                                         where=AND( 
                                                    column.soClass.q.id==self.join_foreign_key(column),
                                                    IN(column.soClass.q.id,ids)
                                                ), groupBy=column.soClass.q.id))

            elif coltype in ('SORelatedJoin', 'SOSQLRelatedJoin'):
                d = (column.intermediateTable,
                     column.joinColumn,
                     column.intermediateTable,
                     column.otherColumn,
                     column.intermediateTable,
                     column.intermediateTable,
                     column.joinColumn,
                    ','.join(['%s'%x for x in ids]),
                     column.intermediateTable,
                     column.joinColumn)
                     
                query = "SELECT %s.%s, Count(%s.%s) FROM %s WHERE %s.%s IN(%s) GROUP BY %s.%s" % d

                
            elif coltype == 'SOSingleJoin':
                alias = self.load_label_column_for_object(column.otherClassName)
                query = conn.sqlrepr( Select( [ column.soClass.q.id,getattr(column.otherClass.q,alias)],
                                                where=AND( 
                                                    column.soClass.q.id==self.join_foreign_key(column),
                                                    IN(column.soClass.q.id,ids)
                                                )))

            if not query: continue
            joins[column.joinMethodName] = conn.queryAll(query)
        return joins

    def foreign_key_alias_value(self,object_name,rows):
        for column in self.foreign_key_columns(object_name): 
            alias = self.load_label_column_for_object(column.foreignKey)
            if alias == 'id':continue
            column_name = column.name.replace('ID','')
            fk_values = self.foreign_key_query(column,alias,[x[column_name] for x in rows])
            for row in rows: row[column_name] = fk_values.get(row[column_name],'')
        return rows

    def foreign_key_query(self,column,alias,ids):
        if not ids: return {}
        sql_object = self.load_object(column.foreignKey)
        conn = sql_object._connection
        query = conn.sqlrepr( Select( [ sql_object.q.id,getattr(sql_object.q,alias) ], 
                                      where=IN(sql_object.q.id,ids) ) )
        fk_values = {}
        for id,alias in conn.queryAll(query): fk_values[str(id)] = self.encode_label( alias )
        return fk_values

    def join_foreign_key(self,column):
        foreign_key = '%sID'% column.joinColumn.split('_')[0]
        return getattr(column.otherClass.q,foreign_key)

    def column_label_options(self,column):
        foreign_key_labels = [{'name':'id','title':'#'}]
        foreign_key_labels.extend(self.column_labels(column.foreignKey))
        return foreign_key_labels

    def column_labels(self,object_name,extended=False):
        cols = []
        sql_object = self.load_object(object_name)
        for column_name in sql_object.sqlmeta.columns:
            column = sql_object.sqlmeta.columns[column_name]
            if sql_object._inheritable and column_name == 'childName':continue
            cols.append({'name':column.name, 'title':self.column_title(column)})
            if isinstance(column, sqlobject.col.SOForeignKey):
                cols[-1]['name'] = cols[-1]['name'].replace('ID','')
            if extended:
                cols[-1]['type']=self.get_column_type(column)
                if isinstance(column, sqlobject.col.SOForeignKey):
                    cols[-1]['column_label'] = self.load_label_column_for_object(column.foreignKey)
                    cols[-1]['label_options']=self.column_label_options(column)
                    cols[-1]['other_class_name'] = column.foreignKey
        return cols

    def join_labels(self,object_name,extended=False):
        cols = []
        sql_object = self.load_object(object_name)
        for col in sql_object.sqlmeta.joins:
            cols.append({'name':col.joinMethodName,'title':''})
            if extended:
                cols[-1]['type']=self.get_column_type(col)
                cols[-1]['other_class_name'] = col.otherClassName
        return cols

    def merge_relation_values(self,rows,relations):
        for field_name in relations:
            for id,value in relations[field_name]:
                if not rows.has_key(id):continue
                rows[id][field_name] = self.encode_label(value)
        keys = rows.keys()
        keys.sort()
        return [rows[x] for x in keys]

    def foreign_key_columns(self,object_name):
        fk = []
        sql_object = self.load_object(object_name)
        for column_name in sql_object.sqlmeta.columns:
            column = sql_object.sqlmeta.columns[column_name]
            if sql_object._inheritable and column_name == 'childName':continue
            if isinstance(column, sqlobject.col.SOForeignKey):fk.append(column)
        return fk

    def field_value(self,result,column):
        value = getattr(result,column.name)
        return self.encode_label(value)

    def fields(self,object_name,result):
        object = self.load_object(object_name)
        props = {}
        props['id'] = result.id
        for column_name in object.sqlmeta.columns:
            column = object.sqlmeta.columns[column_name]
            if object._inheritable and column_name == 'childName':continue
            column_name = column.name
            if isinstance(column, sqlobject.col.SOForeignKey): column_name = column_name.replace('ID','')
            props[column_name] = self.field_value(result, column)
        return props.copy()



from sqlobject import *

__connection__ = connectionForURI("sqlite:///:memory:")
hub = __connection__ 

class Genre(SQLObject):
    name = StringCol()
    artists = RelatedJoin('Artist')

class Artist(SQLObject):
    name = StringCol()
    genres = RelatedJoin('Genre')
    albums = MultipleJoin('Album')

    plays_instruments = RelatedJoin('Instrument', addRemoveName='anInstrument',
                                    joinColumn='artist_id',
                                    otherColumn='plays_instrument_id',
                                    intermediateTable='artist_plays_instrument')
class Album(SQLObject):
    name = StringCol()
    artist = ForeignKey('Artist')
    songs = MultipleJoin('Song')

class Instrument(SQLObject):
    name = StringCol()
    played_by = RelatedJoin( 'Artist', joinColumn='artist_id',
                             otherColumn='plays_instrument_id',
                             intermediateTable='artist_plays_instrument')
    
class Song(SQLObject):
    name = StringCol()
    album = ForeignKey('Album')  

Genre.createTable(ifNotExists=True)
Artist.createTable(ifNotExists=True)
Album.createTable(ifNotExists=True)
Song.createTable(ifNotExists=True)
Instrument.createTable(ifNotExists=True)



import warnings
from turbojson.jsonify import encode
from turbogears.widgets.base import CSSLink, JSLink, CSSSource, JSSource, \
                                    Widget, CoreWD, static, js_location
from turbogears.widgets.forms import SelectionField

try: 
    set
except NameError: 
    from sets import Set as set

__all__ = ["Tabber", "SyntaxHighlighter", "JumpMenu"]

class Tabber(Widget):
    """This widget includes the tabber js and css into your rendered
    page so you can create tabbed divs by assigning them the 'tabber' 
    and 'tabbertab' classes.
    """
    css = [CSSLink(static,"tabber/tabber.css", media="screen")]
    def __init__(self, tabber_options={}, use_cookie=False, hide_on_load=True,
                 *args, **kw):
        super(Tabber, self).__init__(*args, **kw)
        js = []
        # First some sanity-check   
        if use_cookie and (tabber_options.has_key('onLoad') or
                           tabber_options.has_key('onClick')):
            warnings.warn("Cannot use cookies if overriden by "
                          "tabber_options['onClick'] or "
                          "tabber_options['onLoad']. Undefined behavior.")
        # Build the js list in it's correct order
        if use_cookie:
            js.append(JSLink(static, "tabber/tabber_cookie.js"))
        if tabber_options:
            js.append(JSSource("var tabberOptions = %s;" % 
                               encode(tabber_options)))
        if use_cookie:
            js.append(JSSource("""
                               try {
                                   tabberOptions
                               } catch(e){ 
                                   tabberOptions = {};
                               }
                               tabberOptions['onLoad'] = tabber_onload;
                               tabberOptions['onClick'] = tabber_onclick;
                               tabberOptions['cookie'] = 'TGTabber';"""))
        if hide_on_load:
            js.append(JSSource(
"document.write('<style type=\"text/css\">.tabber{display:none;}<\/style>');"))
        js.append(JSLink(static, "tabber/tabber-minimized.js",
                         location=js_location.bodytop))
        self.javascript = js
        

class TabberDesc(CoreWD):
    name = "Tabber"
    for_widget = Tabber()
    template = """<div class="tabber">
        <div class="tabbertab"><h2>Tab 1</h2></div>
        <div class="tabbertab"><h2>Tab 2</h2></div>
        <div class="tabbertab"><h2>Tab 3</h2></div>
        </div>"""

class SyntaxHighlighter(Widget):
    """This widget includes the syntax highlighter js and css into your 
    rendered page to syntax-hightlight textareas named 'code'. The supported
    languages can be listed at the 'languages' __init__ parameter.
    """
    available_langs = set([
        'CSharp',
        'Css',
        'Delphi',
        'Java',
        'JScript',
        'Php',
        'Python',
        'Ruby',
        'Sql',
        'Vb',
        'Xml',
        ])
    css = [CSSLink(static,"sh/SyntaxHighlighter.css")]

    def __init__(self, languages=['Python', 'Xml']):
        javascript = [
            JSLink(static, 'sh/shCore.js', location=js_location.bodybottom)
            ]
        for lang in languages:
            if lang not in self.available_langs:
                raise ValueError, ("Unsupported language %s. Available "
                                   "languages: '%s'" % 
                                   (lang, ', '.join(self.available_langs)))
            source = "sh/shBrush%s.js" % lang
            javascript.append(
                JSLink(static, source, location=js_location.bodybottom)
                )
        javascript.append(
            JSSource(
                "dp.SyntaxHighlighter.HighlightAll('code');",
                location=js_location.bodybottom
                )
            )
        self.javascript = javascript 

class SyntaxHighlighterDesc(CoreWD):
    name = "Syntax Highlighter"
    for_widget = SyntaxHighlighter()
    template = """\
    <textarea name="code" class="py">
        def say_hello():
            print "Hello world!"
    </textarea>"""

class JumpMenu(SelectionField):
    """
    Choose a link from the menu, 
    the page will be redirect to the selected link.
    """
    js = JSSource("""
    <!--
    function TG_jumpMenu(targ,f,restore){ 
      eval(targ+".location='"+f.options[f.selectedIndex].value+"'");
      if (restore) f.selectedIndex=0;
    }
    //-->
    """)
    
    template = """
    <select xmlns:py="http://purl.org/kid/ns#"
        name="${name}"
        class="${field_class}"
        id="${field_id}" 
        onchange="TG_jumpMenu('parent',this,0)"
        py:attrs="attrs"
    >
        <optgroup py:for="group, options in grouped_options"
            label="${group}"
            py:strip="not group"
        >
            <option py:for="value, desc, attrs in options"
                value="${value}"
                py:attrs="attrs"
                py:content="desc"
            />
        </optgroup>
    </select>
    """
    javascript = [js]
    _selected_verb = 'selected'
    params = ["attrs"]
    params_doc = {'attrs' : 'Dictionary containing extra (X)HTML attributes for'
                            ' the select tag'}
    attrs = {}

class JumpMenuDesc(CoreWD):
    name = "Jump Menu"
    for_widget = JumpMenu("your_jump_menu_field",
                          options=[('http://www.python.org', "Python"),
                                   ('http://www.turbogears.org', "TurboGears"),
                                   ('http://www.python.org/pypi', "Cheese Shop"),
                                   ('http://www.pythonware.com/daily/', "Daily Python")],
                          #default='http://www.turbogears.org'
                          )



import turbogears
from datetime import *
from sqlalchemy import *
from sqlalchemy.ext.assignmapper import assign_mapper

from turbogears.visit.api import BaseVisitManager, Visit
from turbogears import config
from turbogears.database import bind_meta_data, metadata, session, get_engine
from turbogears.util import load_class

import logging
log = logging.getLogger("turbogears.identity.savisit")

visit_class = None

class SqlAlchemyVisitManager(BaseVisitManager):

    def __init__(self, timeout):
        global visit_class
        super(SqlAlchemyVisitManager,self).__init__(timeout)
        visit_class_path = config.get("visit.saprovider.model",
                               "turbogears.visit.savisit.TG_Visit")
        visit_class = load_class(visit_class_path)
        bind_meta_data()
        if visit_class is TG_Visit:
            assign_mapper(session.context, visit_class, visits_table)

    def create_model(self):
        '''
        Create the Visit table if it doesn't already exist
        '''
        visit_class.mapper.local_table.create(checkfirst=True)

    def new_visit_with_key(self, visit_key):
        visit = visit_class(visit_key=visit_key,
                        expiry=datetime.now()+self.timeout)
        session.save(visit)
        return Visit(visit_key, True)

    def visit_for_key(self, visit_key):
        '''
        Return the visit for this key or None if the visit doesn't exist or has
        expired.
        '''
        visit = visit_class.lookup_visit(visit_key)
        if not visit:
            return None
        now = datetime.now(visit.expiry.tzinfo)
        if visit.expiry < now:
            return None

        # Visit hasn't expired, extend it
        self.update_visit(visit_key, now+self.timeout)
        return Visit(visit_key, False)

    def update_queued_visits(self, queue):
        # TODO this should be made transactional
        table = visit_class.mapper.mapped_table
        # Now update each of the visits with the most recent expiry
        for visit_key,expiry in queue.items():
            log.info("updating visit (%s) to expire at %s", visit_key,
                      expiry)
            get_engine().execute(table.update(table.c.visit_key==visit_key,
                              values={'expiry': expiry}))

# The Visit table

visits_table = Table('tg_visit', metadata,
    Column('visit_key', String(40), primary_key=True),
    Column('created', DateTime, nullable=False, default=datetime.now),
    Column('expiry', DateTime)
)

class TG_Visit(object):

    def lookup_visit(cls, visit_key):
        return Visit.get(visit_key)
    lookup_visit = classmethod(lookup_visit)

